from __future__ import annotations

from .AFXCommand import AFXCommand
from .AFXKeyword import AFXKeyword

#: Any type is accepted.
AFXTUPLE_TYPE_ANY: int = hash("AFXTUPLE_TYPE_ANY")

#: Element type is the same as the tuple default type.
AFXTUPLE_TYPE_DEFAULT: int = hash("AFXTUPLE_TYPE_DEFAULT")

#: Element is an integer number.
AFXTUPLE_TYPE_INT: int = hash("AFXTUPLE_TYPE_INT")

#: Element is a floating-point number.
AFXTUPLE_TYPE_FLOAT: int = hash("AFXTUPLE_TYPE_FLOAT")

#: Element is a string.
AFXTUPLE_TYPE_STRING: int = hash("AFXTUPLE_TYPE_STRING")

#: Element is True or False.
AFXTUPLE_TYPE_BOOL: int = hash("AFXTUPLE_TYPE_BOOL")

#: Mask for element types.
AFXTUPLE_TYPE_MASK: int = hash("AFXTUPLE_TYPE_MASK")

#: Allow empty values for the element.
AFXTUPLE_ALLOW_EMPTY: int = hash("AFXTUPLE_ALLOW_EMPTY")

#: Always substitute the default for an empty value.
AFXTUPLE_DEFAULT_IF_EMPTY: int = hash("AFXTUPLE_DEFAULT_IF_EMPTY")

#: Evaluate integer and float elements.
AFXTUPLE_EVALUATE: int = hash("AFXTUPLE_EVALUATE")

#: Use tuple default element style.
AFXTUPLE_STYLE_DEFAULT: int = hash("AFXTUPLE_STYLE_DEFAULT")

#: Mask for element styles.
AFXTUPLE_STYLE_MASK: int = hash("AFXTUPLE_STYLE_MASK")


class AFXTupleKeyword(AFXKeyword):
    """This class manages values which are sent as tuples in a command."""

    #: For debugging.
    ID_PRINTSNIPPET: int = hash("ID_PRINTSNIPPET")

    def __init__(
        self,
        command: AFXCommand,
        name: str,
        isRequired: bool = False,
        minLength: int = 0,
        maxLength: int = -1,
        opts: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        command : AFXCommand
            Host command.
        name : str
            Keyword name.
        isRequired : bool
            True if this keyword is a required argument.
        minLength : int
            Minimum (and default) tuple length.
        maxLength : int
            Maximum tuple length (-1 => unlimited).
        opts : int
            Options.
        """

    def equal(self, index: int, a: str, b: str):
        """Returns True if the two tuple element values compare equal (index is not used).

        Parameters
        ----------
        index : int
            Element index (not used).
        a : str
            First value.
        b : str
            Second value.
        """

    def getDefaultStyle(self):
        """Returns the default style for elements."""

    def getDefaultType(self):
        """Returns the default type for elements."""

    def getDefaultValues(self):
        """Returns the default values for this tuple."""

    def getElementStyle(self, index: int):
        """Returns the style of one element. Will never return AFXTUPLE_STYLE_DEFAULT!

        Parameters
        ----------
        index : int
            Element index.
        """

    def getElementType(self, index: int):
        """Returns the type of one element. Will never return AFXTUPLE_TYPE_DEFAULT!

        Parameters
        ----------
        index : int
            Element index.
        """

    def getFormattedValue(self, index: int):
        """Returns the formatted value of the tuple element, suitable for placing in a command. If the element
        has AFXTUPLE_EVALUATE style and its contents are invalid, an exception will be thrown.

        Parameters
        ----------
        index : int
            Element index.
        """

    def getLength(self):
        """Returns the length of the tuple."""

    def getMaxLength(self):
        """Returns the maximum length of this tuple, or -1 for unbounded length."""

    def getMinLength(self):
        """Returns the minimum length of this tuple."""

    def getTypeName(self):
        """Returns the name of the tuple keyword type.

        Implements AFXKeyword.
        """

    def getValue(self, index: int):
        """Returns the value of a tuple element.

        Parameters
        ----------
        index : int
            Element index.
        """

    def getValueAsDouble(self):
        """Returns the keyword's value as a float; returns False upon failure."""

    def getValueAsInt(self):
        """Returns the keyword's value as an integer; returns False upon failure."""

    def getValueAsString(self):
        """Returns the formatted string that represents the current keyword value in a command.

        Implements AFXKeyword.
        """

    def getValueForBlank(self, index: int):
        """Returns the value substituted for blank tuple element.

        Parameters
        ----------
        index : int
            Element index.
        """

    def getValues(self):
        """Returns a string containing values (separated by commas) of the tuple elements."""

    def getValuesForBlanks(self):
        """Returns a string containing values substituted for blanks for the tuple elements."""

    def insertElements(self, index: int, numCols: int):
        """Inserts elements starting at the given index.

        Parameters
        ----------
        index : int
            Starting index.
        numCols : int
            Number of elements to insert.
        """

    def isValueChanged(self):
        """Returns True if the keyword value differs from its previous value.

        Implements AFXKeyword.
        """

    def removeElements(self, index: int, numCols: int):
        """Removes elements starting at the given index.

        Parameters
        ----------
        index : int
            Starting index.
        numCols : int
            Number of elements to remove.
        """

    def setDefaultStyle(self, style: int):
        """Sets the default style for elements.

        Parameters
        ----------
        style : int
            New default element style.
        """

    def setDefaultType(self, type: int):
        """Sets the default type for elements.

        Parameters
        ----------
        type : int
            New default type.
        """

    def setDefaultValues(self, values: str):
        """Sets the default values for this tuple.

        Parameters
        ----------
        values : str
            Sequence string with default values.
        """

    def setElementStyle(self, index: int, style: int):
        """Sets the style of one element.

        Parameters
        ----------
        index : int
            Element index.
        style : int
            New element style.
        """

    def setElementType(self, index: int, type: int):
        """Sets the type of one element.

        Parameters
        ----------
        index : int
            Element index.
        type : int
            New element type.
        """

    def setLengthRange(self, minLength: int, maxLength: int):
        """Sets the range of allowable tuple lengths.

        Parameters
        ----------
        minLength : int
            New minimum length.
        maxLength : int
            New maximum length, or -1 for unbounded length.
        """

    def setMaxLength(self, length: int):
        """Sets the maximum length of this tuple.

        Parameters
        ----------
        length : int
            New maximum length, or -1 for unbounded length.
        """

    def setMinLength(self, length: int):
        """Sets the minimum length of this tuple.

        Parameters
        ----------
        length : int
            New minimum length.
        """

    def setValue(self, index: int, value: str):
        """Sets the value of the tuple element; returns False upon failure.

        Parameters
        ----------
        index : int
            Element index.
        value : str
            New value.
        """

    def setValueForBlank(self, index: int, value: str):
        """Sets the value substituted for a blank tuple element.

        Parameters
        ----------
        index : int
            Element index.
        value : str
            New value.
        """

    def setValues(self, values: str):
        """Sets values for all tuple elements (use commas to separate values within the string).

        Parameters
        ----------
        values : str
            Tuple string with new values.
        """

    def setValuesForBlanks(self, values: str):
        """Sets all values substituted for blanks for the tuple elements.

        Parameters
        ----------
        values : str
            Tuple string with values.
        """

    def setValueToDefault(self, ignoreUnspecified: bool = False):
        """Sets the keyword value to its default.

        Parameters
        ----------
        ignoreUnspecified : bool
            Should ignore if default is an unspecified value.
        """

    def setValueToPrevious(self):
        """Sets the keyword value to its previous value.

        Implements AFXKeyword.
        """

    def syncPreviousValue(self):
        """Sets the keyword's previous value to its current value.

        Implements AFXKeyword.
        """



import auto_all

auto_all.start_all()

from abaqus.PlugInRegistration.AFXApp import *  # noqa
from abaqus.PlugInRegistration.AFXBaseTable import *  # noqa
from abaqus.PlugInRegistration.AFXBoolKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXColorButton import *  # noqa
from abaqus.PlugInRegistration.AFXColorComboBox import *  # noqa
from abaqus.PlugInRegistration.AFXColumnItems import *  # noqa
from abaqus.PlugInRegistration.AFXComboBox import *  # noqa
from abaqus.PlugInRegistration.AFXCommand import *  # noqa
from abaqus.PlugInRegistration.AFXComTableKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXCreateSketchStep import *  # noqa
from abaqus.PlugInRegistration.AFXDataComponent import *  # noqa
from abaqus.PlugInRegistration.AFXDataDialog import *  # noqa
from abaqus.PlugInRegistration.AFXDialog import *  # noqa
from abaqus.PlugInRegistration.AFXDialogStep import *  # noqa
from abaqus.PlugInRegistration.AFXEditSketchStep import *  # noqa
from abaqus.PlugInRegistration.AFXFileDialog import *  # noqa
from abaqus.PlugInRegistration.AFXFileSelectorDialog import *  # noqa
from abaqus.PlugInRegistration.AFXFloatKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXFloatSpinner import *  # noqa
from abaqus.PlugInRegistration.AFXFloatTarget import *  # noqa
from abaqus.PlugInRegistration.AFXFlyoutButton import *  # noqa
from abaqus.PlugInRegistration.AFXFlyoutItem import *  # noqa
from abaqus.PlugInRegistration.AFXForm import *  # noqa
from abaqus.PlugInRegistration.AFXGuiCommand import *  # noqa
from abaqus.PlugInRegistration.AFXGuiMode import *  # noqa
from abaqus.PlugInRegistration.AFXGuiObjectManager import *  # noqa
from abaqus.PlugInRegistration.AFXIntKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXIntTarget import *  # noqa
from abaqus.PlugInRegistration.AFXItemProvider import *  # noqa
from abaqus.PlugInRegistration.AFXKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXList import *  # noqa
from abaqus.PlugInRegistration.AFXListBox import *  # noqa
from abaqus.PlugInRegistration.AFXMainWindow import *  # noqa
from abaqus.PlugInRegistration.AFXMenuCascade import *  # noqa
from abaqus.PlugInRegistration.AFXMenuCommand import *  # noqa
from abaqus.PlugInRegistration.AFXMenuPane import *  # noqa
from abaqus.PlugInRegistration.AFXMenuTitle import *  # noqa
from abaqus.PlugInRegistration.AFXMode import *  # noqa
from abaqus.PlugInRegistration.AFXModuleGui import *  # noqa
from abaqus.PlugInRegistration.AFXNote import *  # noqa
from abaqus.PlugInRegistration.AFXObjectKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXOptionTreeItem import *  # noqa
from abaqus.PlugInRegistration.AFXOptionTreeList import *  # noqa
from abaqus.PlugInRegistration.AFXOrderedPickStep import *  # noqa
from abaqus.PlugInRegistration.AFXPickStep import *  # noqa
from abaqus.PlugInRegistration.AFXProcedure import *  # noqa
from abaqus.PlugInRegistration.AFXProcedureToolsetGui import *  # noqa
from abaqus.PlugInRegistration.AFXProgressBar import *  # noqa
from abaqus.PlugInRegistration.AFXSequenceString import *  # noqa
from abaqus.PlugInRegistration.AFXSlider import *  # noqa
from abaqus.PlugInRegistration.AFXSpinner import *  # noqa
from abaqus.PlugInRegistration.AFXStep import *  # noqa
from abaqus.PlugInRegistration.AFXStringKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXStringTarget import *  # noqa
from abaqus.PlugInRegistration.AFXSymConstKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXTable import *  # noqa
from abaqus.PlugInRegistration.AFXTableKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXTarget import *  # noqa
from abaqus.PlugInRegistration.AFXTextField import *  # noqa
from abaqus.PlugInRegistration.AFXTogglableKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXToolbarGroup import *  # noqa
from abaqus.PlugInRegistration.AFXToolboxGroup import *  # noqa
from abaqus.PlugInRegistration.AFXToolButton import *  # noqa
from abaqus.PlugInRegistration.AFXToolsetGui import *  # noqa
from abaqus.PlugInRegistration.AFXTransition import *  # noqa
from abaqus.PlugInRegistration.AFXTreeTable import *  # noqa
from abaqus.PlugInRegistration.AFXTupleKeyword import *  # noqa
from abaqus.PlugInRegistration.AFXVerticalAligner import *  # noqa
from abaqus.PlugInRegistration.Auxiliary import *
from abaqus.PlugInRegistration.FXApp import *  # noqa
from abaqus.PlugInRegistration.FXArrowButton import *  # noqa
from abaqus.PlugInRegistration.FXBMPIcon import *  # noqa
from abaqus.PlugInRegistration.FXButton import *  # noqa
from abaqus.PlugInRegistration.FXCheckButton import *  # noqa
from abaqus.PlugInRegistration.FXChore import *  # noqa
from abaqus.PlugInRegistration.FXColor import *  # noqa
from abaqus.PlugInRegistration.FXComposite import *  # noqa
from abaqus.PlugInRegistration.FXDataTarget import *  # noqa
from abaqus.PlugInRegistration.FXDialogBox import *  # noqa
from abaqus.PlugInRegistration.FXDrawable import *  # noqa
from abaqus.PlugInRegistration.FXExponent import *  # noqa
from abaqus.PlugInRegistration.FXFileItem import *  # noqa
from abaqus.PlugInRegistration.FXFont import *  # noqa
from abaqus.PlugInRegistration.FXFontDesc import *  # noqa
from abaqus.PlugInRegistration.FXFrame import *  # noqa
from abaqus.PlugInRegistration.FXGIFIcon import *  # noqa
from abaqus.PlugInRegistration.FXGroupBox import *  # noqa
from abaqus.PlugInRegistration.FXHorizontalFrame import *  # noqa
from abaqus.PlugInRegistration.FXHorizontalSeparator import *  # noqa
from abaqus.PlugInRegistration.FXIcon import *  # noqa
from abaqus.PlugInRegistration.FXIconItem import *  # noqa
from abaqus.PlugInRegistration.FXIconList import *  # noqa
from abaqus.PlugInRegistration.FXIconListSortFunc import *  # noqa
from abaqus.PlugInRegistration.FXId import *  # noqa
from abaqus.PlugInRegistration.FXImage import *  # noqa
from abaqus.PlugInRegistration.FXInputHandle import *  # noqa
from abaqus.PlugInRegistration.FXLabel import *  # noqa
from abaqus.PlugInRegistration.FXList import *  # noqa
from abaqus.PlugInRegistration.FXListItem import *  # noqa
from abaqus.PlugInRegistration.FXMainWindow import *  # noqa
from abaqus.PlugInRegistration.FXMatrix import *  # noqa
from abaqus.PlugInRegistration.FXMDIChild import *  # noqa
from abaqus.PlugInRegistration.FXMDIClient import *  # noqa
from abaqus.PlugInRegistration.FXMenubar import *  # noqa
from abaqus.PlugInRegistration.FXMenuButton import *  # noqa
from abaqus.PlugInRegistration.FXMenuCaption import *  # noqa
from abaqus.PlugInRegistration.FXMenuCascade import *  # noqa
from abaqus.PlugInRegistration.FXMenuCommand import *  # noqa
from abaqus.PlugInRegistration.FXMenuPane import *  # noqa
from abaqus.PlugInRegistration.FXMenuSeparator import *  # noqa
from abaqus.PlugInRegistration.FXMenuTitle import *  # noqa
from abaqus.PlugInRegistration.FXMetaClass import *  # noqa
from abaqus.PlugInRegistration.FXObject import *  # noqa
from abaqus.PlugInRegistration.FXObjectList import *  # noqa
from abaqus.PlugInRegistration.FXPacker import *  # noqa
from abaqus.PlugInRegistration.FXPNGIcon import *  # noqa
from abaqus.PlugInRegistration.FXPopup import *  # noqa
from abaqus.PlugInRegistration.FXProgressBar import *  # noqa
from abaqus.PlugInRegistration.FXRadioButton import *  # noqa
from abaqus.PlugInRegistration.FXRGB import *  # noqa
from abaqus.PlugInRegistration.FXRootWindow import *  # noqa
from abaqus.PlugInRegistration.FXScrollArea import *  # noqa
from abaqus.PlugInRegistration.FXScrollbar import *  # noqa
from abaqus.PlugInRegistration.FXScrollWindow import *  # noqa
from abaqus.PlugInRegistration.FXSelector import *  # noqa
from abaqus.PlugInRegistration.FXShell import *  # noqa
from abaqus.PlugInRegistration.FXSplitter import *  # noqa
from abaqus.PlugInRegistration.FXSrollArea import *  # noqa
from abaqus.PlugInRegistration.FXSwitcher import *  # noqa
from abaqus.PlugInRegistration.FXTabBar import *  # noqa
from abaqus.PlugInRegistration.FXTabBook import *  # noqa
from abaqus.PlugInRegistration.FXTabItem import *  # noqa
from abaqus.PlugInRegistration.FXText import *  # noqa
from abaqus.PlugInRegistration.FXTimer import *  # noqa
from abaqus.PlugInRegistration.FXToolbar import *  # noqa
from abaqus.PlugInRegistration.FXTopWindow import *  # noqa
from abaqus.PlugInRegistration.FXTreeItem import *  # noqa
from abaqus.PlugInRegistration.FXTreeList import *  # noqa
from abaqus.PlugInRegistration.FXVerticalFrame import *  # noqa
from abaqus.PlugInRegistration.FXVerticalSeparator import *  # noqa
from abaqus.PlugInRegistration.FXVisual import *  # noqa
from abaqus.PlugInRegistration.FXWindow import *  # noqa
from abaqus.PlugInRegistration.FXXPMIcon import *  # noqa
from abaqus.PlugInRegistration.GUIPlugInCommands import *
from abaqus.PlugInRegistration.KernelPlugInCommands import *
from abaqus.PlugInRegistration.MessageDialogs import *
from abaqus.PlugInRegistration.MessageMaps import *
from abaqusConstants import *

auto_all.end_all()



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import (
    DEMOLD_REGION,
    MAXIMUM,
    ON,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .GeometricRestriction import GeometricRestriction


@abaqus_class_doc
class ShapeDemoldControl(GeometricRestriction):
    """The ShapeDemoldControl object defines a shape demold control geometric restriction. The
    ShapeDemoldControl object is derived from the GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: A VertexArray object of length 2 specifying the demold pull direction. Instead of
    #: through a ConstrainedSketchVertex, each point might be specified through a tuple of coordinates.
    pullDirection: tuple

    #: A Region object specifying the region to which the geometric restriction is applied.
    #: When used with a TopologyTask, there is no default value. When used with a ShapeTask,
    #: the default value is MODEL.
    region: Region

    #: The SymbolicConstant DEMOLD_REGION or a Region object specifying the collision check
    #: region. If the value is DEMOLD_REGION, then the value of **region** is used as both the
    #: demold region and the collision check region. The default value is DEMOLD_REGION.
    collisionCheckRegion: SymbolicConstant = DEMOLD_REGION

    #: None or a DatumCsys object specifying the local coordinate system of the
    #: **pullDirection**. If **csys** = None, the global coordinate system is used. When this member
    #: is queried, it returns an Int indicating the identifier of the DatumCsys. The default
    #: value is None.
    csys: int | None = None

    #: A Float specifying the draw angle. The default value is 0.0.
    drawAngle: float = 0

    #: A SymbolicConstant specifying the rule for assigning point priority. Possible values are
    #: MAXIMUM and MINIMUM. The default value is MAXIMUM.
    #:
    #: .. versionchanged:: 2022
    #:    The attribute ``masterPointDetermination`` was renamed to ``mainPointDetermination``.
    mainPointDetermination: SymbolicConstant = MAXIMUM

    #: A Boolean specifying whether to ignore the geometric restriction in the first design
    #: cycle. The default value is ON.
    presumeFeasibleRegionAtStart: Boolean = ON

    #: A Float specifying the geometric tolerance in the 1-direction. The default value is
    #: 0.01.
    tolerance1: float = 0

    #: A Float specifying the geometric tolerance in the 2-direction. The default value is
    #: 0.01.
    tolerance2: float = 0

    #: A Float specifying the geometric tolerance in the 3-direction. The default value is
    #: 0.01.
    tolerance3: float = 0

    #: A Float specifying the undercut tolerance. The default value is 0.0.
    undercutTolerance: float = 0

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        pullDirection: tuple,
        region: Region,
        collisionCheckRegion: Literal[C.DEMOLD_REGION] = DEMOLD_REGION,
        csys: int | None = None,
        drawAngle: float = 0,
        mainPointDetermination: Literal[C.MINIMUM, C.MAXIMUM] = MAXIMUM,
        presumeFeasibleRegionAtStart: Boolean = ON,
        tolerance1: float = 0,
        tolerance2: float = 0,
        tolerance3: float = 0,
        undercutTolerance: float = 0,
    ):
        """This method creates a ShapeDemoldControl object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].ShapeDemoldControl

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        pullDirection
            A VertexArray object of length 2 specifying the demold pull direction. Instead of
            through a ConstrainedSketchVertex, each point might be specified through a tuple of coordinates.
        region
            A Region object specifying the region to which the geometric restriction is applied.
            When used with a TopologyTask, there is no default value. When used with a ShapeTask,
            the default value is MODEL.
        collisionCheckRegion
            The SymbolicConstant DEMOLD_REGION or a Region object specifying the collision check
            region. If the value is DEMOLD_REGION, then the value of **region** is used as both the
            demold region and the collision check region. The default value is DEMOLD_REGION.
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            **pullDirection**. If **csys** = None, the global coordinate system is used. When this member
            is queried, it returns an Int indicating the identifier of the DatumCsys. The default
            value is None.
        drawAngle
            A Float specifying the draw angle. The default value is 0.0.
        mainPointDetermination
            A SymbolicConstant specifying the rule for assigning point priority. Possible values are
            MAXIMUM and MINIMUM. The default value is MAXIMUM.

            .. versionchanged:: 2022
                The argument ``masterPointDetermination`` was renamed to ``mainPointDetermination``.
        presumeFeasibleRegionAtStart
            A Boolean specifying whether to ignore the geometric restriction in the first design
            cycle. The default value is ON.
        tolerance1
            A Float specifying the geometric tolerance in the 1-direction. The default value is
            0.01.
        tolerance2
            A Float specifying the geometric tolerance in the 2-direction. The default value is
            0.01.
        tolerance3
            A Float specifying the geometric tolerance in the 3-direction. The default value is
            0.01.
        undercutTolerance
            A Float specifying the undercut tolerance. The default value is 0.0.

        Returns
        -------
        ShapeDemoldControl
            A ShapeDemoldControl object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        collisionCheckRegion: Literal[C.DEMOLD_REGION] = DEMOLD_REGION,
        csys: int | None = None,
        drawAngle: float = 0,
        mainPointDetermination: Literal[C.MINIMUM, C.MAXIMUM] = MAXIMUM,
        presumeFeasibleRegionAtStart: Boolean = ON,
        tolerance1: float = 0,
        tolerance2: float = 0,
        tolerance3: float = 0,
        undercutTolerance: float = 0,
    ):
        """This method modifies the ShapeDemoldControl object.

        Parameters
        ----------
        collisionCheckRegion
            The SymbolicConstant DEMOLD_REGION or a Region object specifying the collision check
            region. If the value is DEMOLD_REGION, then the value of **region** is used as both the
            demold region and the collision check region. The default value is DEMOLD_REGION.
        csys
            None or a DatumCsys object specifying the local coordinate system of the
            **pullDirection**. If **csys** = None, the global coordinate system is used. When this member
            is queried, it returns an Int indicating the identifier of the DatumCsys. The default
            value is None.
        drawAngle
            A Float specifying the draw angle. The default value is 0.0.
        mainPointDetermination
            A SymbolicConstant specifying the rule for assigning point priority. Possible values are
            MAXIMUM and MINIMUM. The default value is MAXIMUM.

            .. versionchanged:: 2022
                The argument ``masterPointDetermination`` was renamed to ``mainPointDetermination``.
        presumeFeasibleRegionAtStart
            A Boolean specifying whether to ignore the geometric restriction in the first design
            cycle. The default value is ON.
        tolerance1
            A Float specifying the geometric tolerance in the 1-direction. The default value is
            0.01.
        tolerance2
            A Float specifying the geometric tolerance in the 2-direction. The default value is
            0.01.
        tolerance3
            A Float specifying the geometric tolerance in the 3-direction. The default value is
            0.01.
        undercutTolerance
            A Float specifying the undercut tolerance. The default value is 0.0.
        """
        ...



from __future__ import annotations

from .AFXCommand import AFXCommand
from .AFXKeyword import AFXKeyword
from .constants import FLOAT_DEFAULT


class AFXFloatKeyword(AFXKeyword):
    """This class is designed for the command keywords that have floating-point values."""

    def __init__(
        self,
        command: AFXCommand,
        name: str,
        isRequired: bool = False,
        defaultValue: float = FLOAT_DEFAULT,
        precision: int = 6,
    ):
        """Constructor.

        Parameters
        ----------
        command : AFXCommand
            Host command.
        name : str
            Keyword name.
        isRequired : bool
            True if the keyword is a required argument of the command.
        defaultValue : float
            Default value.
        precision : int
            Precision for converting the keyword's floating-point value to a text string.
        """

    def getPrecision(self):
        """Returns the precision that is used for converting the keyword's floating-point value to a text
        string."""

    def getTypeName(self):
        """Returns the name of the keyword type.

        Implements AFXKeyword.
        """

    def getValue(self):
        """Returns the keyword's current value, or zero if the content expression is invalid."""

    def getValueAsString(self):
        """Returns the text string that represents the keyword's current value.

        Implements AFXKeyword.
        """

    def isValueChanged(self):
        """Returns True if the keyword value differs from its previous value.

        Implements AFXKeyword.
        """

    def setDefaultValue(self, defaultValue: float):
        """Sets the keyword's default value.

        Parameters
        ----------
        defaultValue : float
            Default value.
        """

    def setPrecision(self, precision: int):
        """Sets the precision that is used for converting the keyword's floating-point value to a text string.

        Parameters
        ----------
        precision : int
        """

    def setValue(self, newValue: float):
        """Sets the keyword's current value.

        Parameters
        ----------
        newValue : float
            New value.
        """

    def setValueToDefault(self, ignoreUnspecified: bool = False):
        """Sets the keyword value to its default.

        Parameters
        ----------
        ignoreUnspecified : bool
            Ignore setting the value if the default is unspecified.
        """

    def setValueToPrevious(self):
        """Sets the keyword value to its previous value.

        Implements AFXKeyword.
        """

    def syncPreviousValue(self):
        """Sets the keyword's previous value to its current value.

        Implements AFXKeyword.
        """



"""Kernel plug-in commands register kernel plug-ins in either the **Plug-ins** menu or in a toolbox.

These functions can be accessed by::

    from abaqusGui import getAFXApp
    toolset=getAFXApp().getAFXMainWindow().getPluginToolset()
"""

from typing import Sequence

from ..UtilityAndView.abaqusConstants import ALL, SymbolicConstant
from .FXIcon import FXIcon
from .PluginToolsetBase import PluginToolsetBase


class KernelPluginToolset(PluginToolsetBase):
    def registerKernelMenuButton(
        self,
        moduleName: str,
        functionName: str,
        buttonText: str = "",
        icon: FXIcon | None = None,
        applicableModules: SymbolicConstant | Sequence[str] = ALL,
        version: str = "N/A",
        author: str = "N/A",
        description: str = "N/A",
        helpUrl: str = "N/A",
    ):
        """Registers a kernel plug-in in the Plug-ins menu.

        Parameters
        ----------
        moduleName : str
            A String specifying the name of the module to be imported. The module must contain the function to be executed.
        functionName : str
            A String specifying the name of the function to be executed. The function must be located in moduleName.
        buttonText : str, optional
            A String specifying the text to be displayed in the Plug-ins menu. Use a pipe ( | ) between words to specify
            submenus. The default value is the empty string.
        icon : str, optional
            A FXIcon object specifying an icon to be displayed to the left of the text in the menu. For more information,
            see afxCreateIcon in the “Auxiliary functions” section of this guide. The default value is None.
        applicableModules : str, optional
            The SymbolicConstant ALL or a sequence of one or more Strings specifying the list of modules to which this
            plug-in applies. If a plug-in is not applicable to a module, it will be hidden when the user switches into that
            module. Possible values of the Strings in the sequence are “Part”, “Property”, “Assembly”, “Step”,
            “Interaction”, “Load”, “Mesh”, “Job”, “Visualization”, and “Sketch”. The default value is ALL.
        version : str, optional
            A String specifying the version of the plug-in. The version is displayed in the About Plug-ins dialog box.
            The default value is “N/A”.
        author : str, optional
            A String specifying the author of the plug-in. The author is displayed in the About Plug-ins dialog box.
            The default value is “N/A”.
        description : str, optional
            A String specifying the description of the plug-in. The description is displayed in the About Plug-ins dialog
            box. The default value is “N/A”.
        helpUrl : str, optional
            A String specifying the universal resource locator (URL) that points to the help for this plug-in. This URL can
            be loaded in a web browser from the View button in the About Plug-ins dialog box. The default value is “N/A”.
        """

    def registerKernelToolButton(
        self,
        toolboxName: str,
        moduleName: str,
        functionName: str,
        buttonText: str = "",
        icon: FXIcon | None = None,
        applicableModules: SymbolicConstant | Sequence[str] = ALL,
        version: str = "N/A",
        author: str = "N/A",
        description: str = "N/A",
        helpUrl: str = "N/A",
    ):
        """Registers a kernel plug-in in a toolbox.

        Parameters
        ----------
        toolboxName : str
            A String specifying the name of the toolbox in which the button will be shown. The name appears in the toolbox
            title bar.
        moduleName : str
            A String specifying the name of the module to be imported. The module must contain the function to be executed.
        functionName : str
            A String specifying the name of the function to be executed. The function must be located in _moduleName_.
        buttonText : str, optional
            A String specifying the text to be displayed in the Plug-ins menu. The default value is the empty string.
        icon : str, optional
            A FXIcon object specifying an icon to be displayed to the left of the text in the menu. For more information,
            see afxCreateIcon in the “Auxiliary functions” section of this guide. The default value is None.
        applicableModules : str, optional
            The SymbolicConstant ALL or a sequence of one or more Strings specifying the list of modules to which this
            plug-in applies. If a plug-in is not applicable to a module, it will be hidden when the user switches into that
            module. Possible values of the Strings in the sequence are “Part”, “Property”, “Assembly”, “Step”,
            “Interaction”, “Load”, “Mesh”, “Job”, “Visualization”, and “Sketch”. The default value is ALL.
        version : str, optional
            A String specifying the version of the plug-in. The version is displayed in the About Plug-ins dialog box.
            The default value is “N/A”.
        author : str, optional
            A String specifying the author of the plug-in. The author is displayed in the About Plug-ins dialog box. The
            default value is “N/A”.
        description : str, optional
            A String specifying the description of the plug-in. The description is displayed in the About Plug-ins dialog
            box. The default value is “N/A”.
        helpUrl : str, optional
            A String specifying the universal resource locator (URL) that points to the help for this plug-in. This URL can
            be loaded in a web browser from the View button in the About Plug-ins dialog box. The default value is “N/A”.
        """



from abqpy.decorators import abaqus_class_doc


@abaqus_class_doc
class MdbDataInstance:
    """The MdbDataInstance object instance data. It corresponds to same named part instance with a mesh in the
    cae model.

    .. note::
        This object can be accessed by::

            import visualization
            session.mdbData[name].instances[i]
    """

    #: A String specifying the instance name. This attribute is read-only.
    name: str = ""



from __future__ import annotations

from .FXApp import FXApp
from .FXFontDesc import FXFontDesc
from .FXId import FXId

#: Default pitch.
FONTPITCH_DEFAULT: int = hash("FONTPITCH_DEFAULT")

#: Fixed pitch, mono-spaced.
FONTPITCH_FIXED: int = hash("FONTPITCH_FIXED")

#: Variable pitch, proportional spacing.
FONTPITCH_VARIABLE: int = hash("FONTPITCH_VARIABLE")

#: Don't care which font.
FONTHINT_DONTCARE: int = hash("FONTHINT_DONTCARE")

#: Fancy fonts.
FONTHINT_DECORATIVE: int = hash("FONTHINT_DECORATIVE")

#: Monospace typewriter font.
FONTHINT_MODERN: int = hash("FONTHINT_MODERN")

#: Variable width times-like font, serif.
FONTHINT_ROMAN: int = hash("FONTHINT_ROMAN")

#: Script or cursive.
FONTHINT_SCRIPT: int = hash("FONTHINT_SCRIPT")

#: Helvetica/swiss type font, sans-serif.
FONTHINT_SWISS: int = hash("FONTHINT_SWISS")

#: System font.
FONTHINT_SYSTEM: int = hash("FONTHINT_SYSTEM")

#: X11 Font string.
FONTHINT_X11: int = hash("FONTHINT_X11")

#: Scalable fonts.
FONTHINT_SCALABLE: int = hash("FONTHINT_SCALABLE")

#: Polymorphic fonts.
FONTHINT_POLYMORPHIC: int = hash("FONTHINT_POLYMORPHIC")

#: Don't care about slant.
FONTSLANT_DONTCARE: int = hash("FONTSLANT_DONTCARE")

#: Regular straight up.
FONTSLANT_REGULAR: int = hash("FONTSLANT_REGULAR")

#: Italics.
FONTSLANT_ITALIC: int = hash("FONTSLANT_ITALIC")

#: Oblique slant.
FONTSLANT_OBLIQUE: int = hash("FONTSLANT_OBLIQUE")

#: Reversed italic.
FONTSLANT_REVERSE_ITALIC: int = hash("FONTSLANT_REVERSE_ITALIC")

#: Reversed oblique.
FONTSLANT_REVERSE_OBLIQUE: int = hash("FONTSLANT_REVERSE_OBLIQUE")

#: Don't care character encoding.
FONTENCODING_DEFAULT: int = hash("FONTENCODING_DEFAULT")

#: Cyrillic (almost obsolete).
FONTENCODING_ISO_8859_5: int = hash("FONTENCODING_ISO_8859_5")

#: Russian.
FONTENCODING_KOI8_R: int = hash("FONTENCODING_KOI8_R")

#: Ukrainian.
FONTENCODING_KOI8_U: int = hash("FONTENCODING_KOI8_U")

#: Latin 1 (West European).
FONTENCODING_LATIN1: int = hash("FONTENCODING_LATIN1")

#: Latin 2 (East European).
FONTENCODING_LATIN2: int = hash("FONTENCODING_LATIN2")

#: Latin 3 (South European).
FONTENCODING_LATIN3: int = hash("FONTENCODING_LATIN3")

#: Latin 4 (North European).
FONTENCODING_LATIN4: int = hash("FONTENCODING_LATIN4")

#: Latin 5 (Turkish).
FONTENCODING_LATIN5: int = hash("FONTENCODING_LATIN5")

#: Latin 6 (Nordic).
FONTENCODING_LATIN6: int = hash("FONTENCODING_LATIN6")

#: Latin 7 (Baltic Rim).
FONTENCODING_LATIN7: int = hash("FONTENCODING_LATIN7")

#: Latin 8 (Celtic).
FONTENCODING_LATIN8: int = hash("FONTENCODING_LATIN8")

#: Latin 9 AKA Latin 0.
FONTENCODING_LATIN9: int = hash("FONTENCODING_LATIN9")

#: Latin 10.
FONTENCODING_LATIN10: int = hash("FONTENCODING_LATIN10")

#: Latin 1.
FONTENCODING_USASCII: int = hash("FONTENCODING_USASCII")

#: Latin 1 (West European).
FONTENCODING_WESTEUROPE: int = hash("FONTENCODING_WESTEUROPE")

#: Latin 2 (East European).
FONTENCODING_EASTEUROPE: int = hash("FONTENCODING_EASTEUROPE")

#: Latin 3 (South European).
FONTENCODING_SOUTHEUROPE: int = hash("FONTENCODING_SOUTHEUROPE")

#: Latin 4 (North European).
FONTENCODING_NORTHEUROPE: int = hash("FONTENCODING_NORTHEUROPE")

#: Cyrillic.
FONTENCODING_CYRILLIC: int = hash("FONTENCODING_CYRILLIC")

#: Cyrillic.
FONTENCODING_RUSSIAN: int = hash("FONTENCODING_RUSSIAN")

#: Arabic.
FONTENCODING_ARABIC: int = hash("FONTENCODING_ARABIC")

#: Greek.
FONTENCODING_GREEK: int = hash("FONTENCODING_GREEK")

#: Hebrew.
FONTENCODING_HEBREW: int = hash("FONTENCODING_HEBREW")

#: Latin 5 (Turkish).
FONTENCODING_TURKISH: int = hash("FONTENCODING_TURKISH")

#: Latin 6 (Nordic).
FONTENCODING_NORDIC: int = hash("FONTENCODING_NORDIC")

#: Thai.
FONTENCODING_THAI: int = hash("FONTENCODING_THAI")

#: Latin 7 (Baltic Rim).
FONTENCODING_BALTIC: int = hash("FONTENCODING_BALTIC")

#: Latin 8 (Celtic).
FONTENCODING_CELTIC: int = hash("FONTENCODING_CELTIC")

#: Don't care about weight.
FONTWEIGHT_DONTCARE: int = hash("FONTWEIGHT_DONTCARE")

#: Thin.
FONTWEIGHT_THIN: int = hash("FONTWEIGHT_THIN")

#: Extra light.
FONTWEIGHT_EXTRALIGHT: int = hash("FONTWEIGHT_EXTRALIGHT")

#: Light.
FONTWEIGHT_LIGHT: int = hash("FONTWEIGHT_LIGHT")

#: Normal or regular weight.
FONTWEIGHT_NORMAL: int = hash("FONTWEIGHT_NORMAL")

#: Normal or regular weight.
FONTWEIGHT_REGULAR: int = hash("FONTWEIGHT_REGULAR")

#: Medium bold face.
FONTWEIGHT_MEDIUM: int = hash("FONTWEIGHT_MEDIUM")

#: Demi bold face.
FONTWEIGHT_DEMIBOLD: int = hash("FONTWEIGHT_DEMIBOLD")

#: Bold face.
FONTWEIGHT_BOLD: int = hash("FONTWEIGHT_BOLD")

#: Extra.
FONTWEIGHT_EXTRABOLD: int = hash("FONTWEIGHT_EXTRABOLD")

#: Heavy.
FONTWEIGHT_HEAVY: int = hash("FONTWEIGHT_HEAVY")

#: Black.
FONTWEIGHT_BLACK: int = hash("FONTWEIGHT_BLACK")

#: Don't care about set width.
FONTSETWIDTH_DONTCARE: int = hash("FONTSETWIDTH_DONTCARE")

#: Ultra condensed printing.
FONTSETWIDTH_ULTRACONDENSED: int = hash("FONTSETWIDTH_ULTRACONDENSED")

#: Extra condensed.
FONTSETWIDTH_EXTRACONDENSED: int = hash("FONTSETWIDTH_EXTRACONDENSED")

#: Condensed.
FONTSETWIDTH_CONDENSED: int = hash("FONTSETWIDTH_CONDENSED")

#: Narrow.
FONTSETWIDTH_NARROW: int = hash("FONTSETWIDTH_NARROW")

#: Compressed.
FONTSETWIDTH_COMPRESSED: int = hash("FONTSETWIDTH_COMPRESSED")

#: Semi-condensed.
FONTSETWIDTH_SEMICONDENSED: int = hash("FONTSETWIDTH_SEMICONDENSED")

#: Medium printing.
FONTSETWIDTH_MEDIUM: int = hash("FONTSETWIDTH_MEDIUM")

#: Normal printing.
FONTSETWIDTH_NORMAL: int = hash("FONTSETWIDTH_NORMAL")

#: Regular printing.
FONTSETWIDTH_REGULAR: int = hash("FONTSETWIDTH_REGULAR")

#: Semi expanded.
FONTSETWIDTH_SEMIEXPANDED: int = hash("FONTSETWIDTH_SEMIEXPANDED")

#: Expanded.
FONTSETWIDTH_EXPANDED: int = hash("FONTSETWIDTH_EXPANDED")

#: Wide.
FONTSETWIDTH_WIDE: int = hash("FONTSETWIDTH_WIDE")

#: Extra expanded.
FONTSETWIDTH_EXTRAEXPANDED: int = hash("FONTSETWIDTH_EXTRAEXPANDED")

#: Ultra expanded.
FONTSETWIDTH_ULTRAEXPANDED: int = hash("FONTSETWIDTH_ULTRAEXPANDED")


class FXFont(FXId):
    """Font class."""

    def __init__(self, a: FXApp, nm: str):
        """Construct a font with given X11 font string.

        Parameters
        ----------
        a : FXApp

        nm : str
        """

    def create(self):
        """Create the font.

        Reimplemented from FXId.
        """

    def destroy(self):
        """Destroy the font.

        Reimplemented from FXId.
        """

    def detach(self):
        """Detach the font.

        Reimplemented from FXId.
        """

    def getEncoding(self):
        """Get character set encoding."""

    def getFontAscent(self):
        """Ascent from baseline."""

    def getFontDesc(self, fontdesc: FXFontDesc):
        """Get font description.

        Parameters
        ----------
        fontdesc : FXFontDesc
        """

    def getFontDescent(self):
        """Descent from baseline."""

    def getFontHeight(self):
        """Height of highest character in font."""

    def getFontLeading(self):
        """Get font leading [that is lead-ing as in Pb!]."""

    def getFontSpacing(self):
        """Get font line spacing."""

    def getFontWidth(self):
        """Width of widest character in font."""

    def getHints(self):
        """Get hints."""

    def getMaxChar(self):
        """Get last character glyph in font."""

    def getMinChar(self):
        """Get first character glyph in font."""

    def getName(self):
        """Get face name."""

    def getSetWidth(self):
        """Get setwidth."""

    def getSize(self):
        """Get size in deci-points."""

    def getSlant(self):
        """Get slant."""

    def getTextHeight(self, text: str, n: int):
        """Calculate height of given text in this font.

        Parameters
        ----------
        text : str
            The string whose height is being evaluated.
        n : int
            The number of characters in 'text,' starting from the left end, for which the height will be returned.
        """

    def getTextWidth(self, text: str, n: int):
        """Calculate width of given text in this font.

        Parameters
        ----------
        text : str
            The string whose width is being evaluated.
        n : int
            The number of characters in 'text,' starting from the left end, for which the width will be returned.
        """

    def getWeight(self):
        """Get font weight."""

    def hasChar(self, ch: int):
        """See if font has glyph for ch.

        Parameters
        ----------
        ch : int
        """

    def isFontMono(self):
        """Find out if the font is monotype or proportional."""

    def leftBearing(self, ch: str):
        """Left bearing.

        Parameters
        ----------
        ch : str
        """

    def rightBearing(self, ch: str):
        """Right bearing.

        Parameters
        ----------
        ch : str
        """

    def setFontDesc(self, fontdesc: FXFontDesc):
        """Change font description.

        Parameters
        ----------
        fontdesc : FXFontDesc
        """



from __future__ import annotations

from .FXApp import FXApp
from .FXColor import FXColor
from .FXIcon import FXIcon
from .FXRGB import FXRGB


class FXXPMIcon(FXIcon):
    """X Pixmap icon."""

    def __init__(
        self, a: FXApp, pix: str = "None", clr: FXColor = FXRGB(192, 192, 192), opts: int = 0, w: int = 1, h: int = 1
    ):
        """Construct icon from compiled-in X Pixmap format.

        Parameters
        ----------
        a : FXApp

        pix : str

        clr : FXColor

        opts : int

        w : int

        h : int
        """



from __future__ import annotations

from .FXColor import FXColor
from .FXComposite import FXComposite
from .FXFont import FXFont
from .FXIcon import FXIcon
from .FXMDIClient import FXMDIClient
from .FXMenuPane import FXMenuPane
from .FXPopup import FXPopup

#: Normal display mode.
MDI_NORMAL: int = hash("MDI_NORMAL")

#: Window appears maximized.
MDI_MAXIMIZED: int = hash("MDI_MAXIMIZED")

#: Window is iconified or minimized.
MDI_MINIMIZED: int = hash("MDI_MINIMIZED")


class FXMDIChild(FXComposite):
    """The MDI child window contains the application work area in a Multiple Document Interface application."""

    def __init__(
        self,
        p: FXMDIClient,
        name: str,
        ic: FXIcon | None = None,
        mn: FXMenuPane | None = None,
        opts: int = 0,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
    ):
        """Construct MDI Child window with given name and icon.

        Parameters
        ----------
        p : FXMDIClient

        name : str

        ic : FXIcon | None

        mn : FXMenuPane | None

        opts : int

        x : int

        y : int

        w : int

        h : int
        """

    def canFocus(self):
        """MDI Child can receive focus.

        Reimplemented from FXWindow.
        """

    def contentWindow(self):
        """Return content window."""

    def create(self):
        """Create window.

        Reimplemented from FXComposite.
        """

    def detach(self):
        """Detach window.

        Reimplemented from FXComposite.
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXComposite.
        """

    def getDefaultWidth(self):
        """Compute default size.

        Reimplemented from FXComposite.
        """

    def getFont(self):
        """Get title font."""

    def getHiliteColor(self):
        """Get colors."""

    def getIconX(self):
        """Return iconified position."""

    def getMDINext(self):
        """Get next MDI Child."""

    def getMDIPrev(self):
        """Get previous MDI Child."""

    def getNormalX(self):
        """Return normal (restored) position."""

    def getTitle(self):
        """Get current title."""

    def getWindowIcon(self):
        """Get window icon."""

    def getWindowMenu(self):
        """Get window menu."""

    def isMaximized(self):
        """Return True if maximized."""

    def isMinimized(self):
        """Return True if minimized."""

    def maximize(self, notify: bool = False):
        """Maximize MDI Child.

        Parameters
        ----------
        notify : bool
        """

    def minimize(self, notify: bool = False):
        """Minimize/iconify MDI Child.

        Parameters
        ----------
        notify : bool
        """

    def move(self, x: int, y: int):
        """Move this window to the specified position in the parent's coordinates. Reimplemented from FXWindow.

        Parameters
        ----------
        x : int

        y : int
        """

    def position(self, x: int, y: int, w: int, h: int):
        """Move and resize this window in the parent's coordinates. Reimplemented from FXWindow.

        Parameters
        ----------
        x : int

        y : int

        w : int

        h : int
        """

    def resize(self, w: int, h: int):
        """Resize this window to the specified width and height. Reimplemented from FXWindow.

        Parameters
        ----------
        w : int

        h : int
        """

    def restore(self, notify: bool = False):
        """Restore MDI Child to normal.

        Parameters
        ----------
        notify : bool
        """

    def setFont(self, fnt: FXFont):
        """Set title font.

        Parameters
        ----------
        fnt : FXFont
        """

    def setHiliteColor(self, clr: FXColor):
        """Change colors.

        Parameters
        ----------
        clr : FXColor
        """

    def setIconX(self, x: int):
        """Change iconified position.

        Parameters
        ----------
        x : int
        """

    def setNormalX(self, x: int):
        """Change normal (restored) position.

        Parameters
        ----------
        x : int
        """

    def setTitle(self, name: str):
        """Change MDI Child's title.

        Parameters
        ----------
        name : str
        """

    def setWindowIcon(self, icon: FXIcon):
        """Set window icon.

        Parameters
        ----------
        icon : FXIcon
        """

    def setWindowMenu(self, menu: FXPopup):
        """Set window menu.

        Parameters
        ----------
        menu : FXPopup
        """



"""This section lists various auxiliary GUI toolkit functions."""

from __future__ import annotations

from typing import Callable

from .AFXApp import AFXApp
from .AFXModuleGui import AFXModuleGui
from .FXBMPIcon import FXBMPIcon
from .FXFont import FXFont
from .FXGIFIcon import FXGIFIcon
from .FXObject import FXObject
from .FXPNGIcon import FXPNGIcon
from .FXXPMIcon import FXXPMIcon


def addExitCallback(callback: Callable):
    """Registers a callback function to be called when the application is about to exit.

    Parameters
    ----------
    callback : Callable
        The function to be called when the application is about to exit.
    """


def afxCreateIcon(fileName: str) -> FXBMPIcon | FXGIFIcon | FXPNGIcon | FXXPMIcon:  # type: ignore
    """Returns an icon created by reading the specified file, which can be in one of these formats: BMP, GIF,
    PNG, XPM. The file format is assumed from the file extension (which is not case sensitive). Returns 0 if the
    file cannot be opened.

    Parameters
    ----------
    fileName : str
        File name.
    """
    ...


def afxCreateBMPIcon(fileName: str) -> FXBMPIcon:  # type: ignore
    """Returns an icon created by reading the specified file in BMP format. Returns 0 if the file cannot be
    opened.

    Parameters
    ----------
    fileName : str
        File name.
    """
    ...


def afxCreateGIFIcon(fileName: str) -> FXGIFIcon:  # type: ignore
    """Returns an icon created by reading the specified file in GIF format. Returns 0 if the file cannot be
    opened.

    Parameters
    ----------
    fileName : str
        File name.
    """
    ...


def afxCreatePNGIcon(fileName: str) -> FXPNGIcon:  # type: ignore
    """Returns an icon created by reading the specified file in PNG format. Returns 0 if the file cannot be
    opened.

    Parameters
    ----------
    fileName : str
        File name.
    """
    ...


def afxCreateXPMIcon(fileName: str) -> FXXPMIcon:  # type: ignore
    """Returns an icon created by reading the specified file in XPM format. Returns 0 if the file cannot be
    opened.

    Parameters
    ----------
    fileName : str
        File name.
    """
    ...


def afxGetIcon(fileName: str, size: str):
    """Enables you to use Abaqus/CAE icons in your customization. Set the _size_ argument to 1 for normal-sized
    icons or to 0 for small icons.

    Parameters
    ----------
    fileName : str
        File name.
    size : str
        Icon size.
    """


def displayURL(url: str):
    """Displays the specified URL in a web browser. Returns the status of the call. This call will use an open web browser if there is one. This method can be accessed via webBrowser.displayURL from module uti. See also [openWithURL](https://help.3ds.com/2023/English/DSSIMULIA_Established/SIMACAEGUIRefHtml/pt01ch01gob125.htm?contextscope=all#gui-auxiliary-openwithurl).

    Parameters
    ----------
    url : str
        The URL of the page to be displayed.
    """


def getAFXApp() -> AFXApp:  # type: ignore
    """Returns the application object."""
    ...


def getAFXAliasMap():
    """Returns the alias map."""


FONT_PROPORTIONAL: int = hash("FONT_PROPORTIONAL")
FONT_MONOSPACE: int = hash("FONT_MONOSPACE")
FONT_REGULAR: int = hash("FONT_REGULAR")
FONT_BOLD: int = hash("FONT_BOLD")
FONT_ITALIC: int = hash("FONT_ITALIC")
FONT_BASE: int = hash("FONT_BASE")
FONT_SMALL: int = hash("FONT_SMALL")


def getAFXFont(opts: int = FONT_PROPORTIONAL) -> FXFont:  # type: ignore
    """Returns the specified font.

    Parameters
    ----------
    opts : int
        Type of font to get. Possible choices are FONT_PROPORTIONAL, FONT_MONOSPACE, FONT_REGULAR, FONT_BOLD, FONT_ITALIC, FONT_BASE, or FONT_SMALL.
    """
    ...


def afxGetColorHexSpecFromID(colorId: int) -> str:  # type: ignore
    """Returns the equivalent hex string for the specified color index.

    Parameters
    ----------
    colorId : int
        The index of the color to be converted to a hex string. For example, green would be FXRGB(0, 255,0). See the Colors appendix of the Abaqus GUI Toolkit User's Guide for details.
    """
    ...


def afxGetColorHexSpecFromName(colorName: str) -> str:  # type: ignore
    """Returns the equivalent hex string for the specified color name.

    Parameters
    ----------
    colorName : str
        The name of the color to be converted to a hex string. For example, 'Red'. See the Colors appendix of the Abaqus GUI Toolkit User's Guide for details.
    """
    ...


def getCurrentContext() -> dict:  # type: ignore
    """Returns the current GUI context dictionary, which contains the following keys: _mdbName_, _viewportName_,
    _objectPath_, _objectType_, _modelName_, and _moduleName_.

    You can be notified of context changes by using the [registerCurrentContext](https://help.3ds.com/2023/English/DSSIMULIA_Established/SIMACAEGUIRefHtml/pt01ch01gob125.htm?contextscope=all#gui-auxiliary-registercurrentcontext) function.
    """
    ...


def getCurrentModuleGui() -> AFXModuleGui:  # type: ignore
    """Returns the current moduleGui object.

    You can call that object's getModuleName method to check its name.
    """
    ...


def getCursorPosition() -> tuple[bool, float, float, int]:  # type: ignore
    """Returns a tuple of status,x,y,buttonState.

    The status (TRUE or FALSE) indicates the success of the call. The x and y values represent the position
    of the cursor in the window's coordinate system (origin is in the upper left, positive Y points
    downward). This method is defined in FXWindow, so it may be called on any object derived from FXWindow.
    """
    ...


def getDisplayedObjectType() -> str:  # type: ignore
    """Returns the type of the object displayed in the current viewport.

    Possible return values are: PART, ASSEMBLY, ODB, XY_PLOT, SKETCH, or None.
    """
    ...


def getSeparator(parent, count: int):
    """Returns the nth (specified by the count argument) separator widget of the parent.

    Parameters
    ----------
    parent : Widget
        The widget to be searched (children are not traversed).
    count : int
        The number of the separator (e.g. specify 2 to get the second separator).
    """


def getWidgetFromText(parent, text: str) -> FXObject:  # type: ignore
    """Returns a widget whose label or tip text matches the specified text and is also a child of the specified
    widget.

    Parameters
    ----------
    parent : Widget
        The widget at which the search begins (children are traversed too).
    text : str
        The text to be matched (must be an exact match).
    """
    ...


def openWithURL(url: str):
    """Displays the specified URL in a web browser. Returns the status of the call. This call will always open a new web browser. This method can be accessed via webBrowser.openWithURL from module uti. See also [displayURL](https://help.3ds.com/2023/English/DSSIMULIA_Established/SIMACAEGUIRefHtml/pt01ch01gob125.htm?contextscope=all#gui-auxiliary-displayurl).

    Parameters
    ----------
    url : str
        The URL of the page to be displayed.
    """


def registerCurrentContext(callbackFunction: Callable):
    """Registers a query on the current context. The specified callback function will be invoked when the current viewport or displayed object changes. Use the getCurrentContext method to get the values of the current objects. Note that [unregisterCurrentContext](https://help.3ds.com/2023/English/DSSIMULIA_Established/SIMACAEGUIRefHtml/pt01ch01gob125.htm?contextscope=all#gui-auxiliary-unregistercurrentcontext) should be called with the same argument to unregister the query when it is no longer needed.

    Parameters
    ----------
    callbackFunction : Callable
        Function to be called when the current context changes.
    """


def removeExitCallback(callback: Callable):
    """Unregisters a callback function to be called when the application is about to exit. The function must
    have been previously registered using addExitCallback.

    Parameters
    ----------
    callback : Callable
        The function to be removed from the callback list.
    """


def sendCommand(writeToJournal: bool, command: str, writeToReplay: bool = True):
    """Sends a command string to the kernel. **Note:** The _writeToJournal_ argument should not usually be
    necessary. If the command is a built-in Abaqus Scripting Interface command, it will be written to the
    journal file automatically. If the command is not a built-in Abaqus Scripting Interface command but it
    changes the mdb using built-in Abaqus Scripting Interface commands, the changes to the mdb will be written
    to the journal file automatically. However, if the command is not a built-in Abaqus Scripting Interface
    command and it makes changes to the mdb that would not otherwise be written to the journal file, the command
    it should use the journalMethodCall function to write it to the journal file.

    Parameters
    ----------
    writeToJournal : bool
        If True, the command will be written to the journal file.
    command : str
        Command string.
    writeToReplay : bool
        If True, the command will be written to the replay file.
    """


def setCurrentModel(modelName: str):
    """Sets the current model to the specified model name.

    Parameters
    ----------
    modelName : str
        The name of the model to be made current.
    """


def setSwitchModuleHook(callbackFunction: Callable):
    """Sets a function that will be called whenever the user switches into a GUI module. When the user switches
    into a GUI module, the specified function will be called, passing it the name (shown in the Module combo
    box) of the new module. Note that the setSwitchModuleHook function does not take keyword arguments and, if
    necessary, it may be called multiple times.

    Parameters
    ----------
    callbackFunction : Callable
        The function to be called when a GUI module is switched into. Note that you cannot issue a kernel command from within this function or it will cause the application to lock up.
    """


def shutdown():
    """Exits Abaqus/CAE.

    This is equivalent to selecting File->Exit.
    """


def switchModule(moduleName: str):
    """Switches into a GUI module.

    Parameters
    ----------
    moduleName : str
        Module to switch into.
    """


def unregisterCurrentContext(callbackFunction: Callable):
    """Unregisters a query on the current context. The specified callback function should be the same argument
    that was used to register the query.

    Parameters
    ----------
    callbackFunction : Callable
        Function to be called when the current context changes.
    """



from __future__ import annotations

from .AFXDataComponent import AFXDataComponent
from .FXComposite import FXComposite
from .FXObject import FXObject
from .FXPacker import FXPacker

#: Slider shown horizontally.
AFXSLIDER_HORIZONTAL: int = hash("AFXSLIDER_HORIZONTAL")

#: Slider shown vertically.
AFXSLIDER_VERTICAL: int = hash("AFXSLIDER_VERTICAL")

#: Slider has arrow head pointing up.
AFXSLIDER_ARROW_UP: int = hash("AFXSLIDER_ARROW_UP")

#: Slider has arrow head pointing down.
AFXSLIDER_ARROW_DOWN: int = hash("AFXSLIDER_ARROW_DOWN")

#: Slider has arrow head pointing left.
AFXSLIDER_ARROW_LEFT: int = hash("AFXSLIDER_ARROW_LEFT")

#: Slider has arrow head pointing right.
AFXSLIDER_ARROW_RIGHT: int = hash("AFXSLIDER_ARROW_RIGHT")

#: Slider is inside the slot rather than overhanging.
AFXSLIDER_INSIDE_BAR: int = hash("AFXSLIDER_INSIDE_BAR")

#: Show slider value.
AFXSLIDER_SHOW_VALUE: int = hash("AFXSLIDER_SHOW_VALUE")

#: Show slider above its title.
AFXSLIDER_ABOVE_TITLE: int = hash("AFXSLIDER_ABOVE_TITLE")

#: Show slider after its title.
AFXSLIDER_AFTER_TITLE: int = hash("AFXSLIDER_AFTER_TITLE")

#: Default slider options--slider is horizontal, inside the slot, and shown above its title label.
AFXSLIDER_NORMAL: int = hash("AFXSLIDER_NORMAL")


class AFXSlider(FXPacker, AFXDataComponent):
    """This class provides a slider, which allows the user to specify a value by dragging its value
    indicator."""

    #: ID for the slider.
    ID_SLIDER: int = hash("ID_SLIDER")

    #: Last ID for this class.
    ID_LAST: int = hash("ID_LAST")

    def __init__(
        self,
        p: FXComposite,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = AFXSLIDER_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = 0,
        pr: int = 0,
        pt: int = 0,
        pb: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite
            Parent widget.
        tgt : FXObject | None
            Message target.
        sel : int
            Message ID.
        opts : int
            Options and hints.
        x : int
            X coordinate of origin.
        y : int
            Y coordinate of origin.
        w : int
            Width of the widget.
        h : int
            Height of the widget.
        pl : int
            Left padding (margin).
        pr : int
            Right padding (margin).
        pt : int
            Top padding (margin).
        pb : int
            Bottom padding (margin).
        """

    def canFocus(self):
        """Returns True because a slider can receive focus.

        Reimplemented from FXWindow.
        """

    def disable(self):
        """Disables the slider.

        Reimplemented from FXWindow.
        """

    def enable(self):
        """Enables the slider.

        Reimplemented from FXWindow.
        """

    def getDecimalPlaces(self):
        """Returns the number of decimal points displayed."""

    def getDefaultHeight(self):
        """Returns the default height.

        Reimplemented from FXPacker.
        """

    def getDefaultWidth(self):
        """Returns the default width.

        Reimplemented from FXPacker.
        """

    def getIncrement(self):
        """Returns the slider's auto-increment/decrement value."""

    def getMaxLabelText(self):
        """Returns the maximum label's text."""

    def getMinLabelText(self):
        """Returns the minimum label's text."""

    def getRange(self):
        """Returns a sequence of ints (low, high) representing the widget's allowable minimum and maximum
        values."""

    def getSliderStyle(self):
        """Returns the slider's style."""

    def getTipText(self):
        """Returns the slider's tip text."""

    def getTitleLabelJustify(self):
        """Returns the title label's justification mode."""

    def getTitleLabelText(self):
        """Returns the title label's text."""

    def getValue(self):
        """Returns the slider's value."""

    def recalc(self):
        """Recalculates the slider.

        Redefined to handle slider movement. Reimplemented from FXWindow.
        """

    def setDecimalPlaces(self, dp: int):
        """Sets the number of decimal points displayed.

        Parameters
        ----------
        dp : int
            Number of decimal places.
        """

    def setIncrement(self, inc: int):
        """Sets the slider's auto-increment/decrement value.

        Parameters
        ----------
        inc : int
            Increment.
        """

    def setMaxLabelText(self, text: str):
        """Sets the maximum label's text.

        Parameters
        ----------
        text : str
            Max label text.
        """

    def setMinLabelText(self, text: str):
        """Sets the minimum label's text.

        Parameters
        ----------
        text : str
            Min label text.
        """

    def setRange(self, lo: int, hi: int):
        """Sets the slider's maximum and minimum values.

        Parameters
        ----------
        lo : int
            Minimum value.
        hi : int
            Maximum value.
        """

    def setSliderStyle(self, style: int):
        """Sets the slider's style.

        Parameters
        ----------
        style : int
            Style flag.
        """

    def setTipText(self, text: str):
        """Sets the slider's tip text.

        Parameters
        ----------
        text : str
            Tip text.
        """

    def setTitleLabelJustify(self, mode: int):
        """Sets the title label's justification mode.

        Parameters
        ----------
        mode : int
            Justification mode.
        """

    def setTitleLabelText(self, text: str):
        """Sets the title label's text.

        Parameters
        ----------
        text : str
            Title text.
        """

    def setValue(self, value: int):
        """Sets the slider's value.

        Parameters
        ----------
        value : int
            Value.
        """

    def show(self):
        """Shows the slider.

        Reimplemented from FXWindow.
        """



from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...BasicGeometry.Vertex import Vertex
from ..ConstrainedSketchBase import ConstrainedSketchBase
from ..ConstrainedSketchGeometry.ConstrainedSketchGeometry import (
    ConstrainedSketchGeometry,
)
from .ConstrainedSketchConstraint import ConstrainedSketchConstraint


@abaqus_class_doc
class ConstrainedSketchConstraintModel(ConstrainedSketchBase):
    """A ConstrainedSketch object contains the entities that are used to create a sketch. The objects include
    ConstrainedSketchGeometry objects contained in the ConstrainedSketchGeometry Repository, such as Line, Arc,
    and Spline. ConstrainedSketchVertex, ConstrainedSketchDimension, ConstrainedSketchConstraint, and
    ConstrainedSketchParameter objects are contained in their respective repositories.

    .. note::
        This object can be accessed by::

            import sketch
            mdb.models[name].sketches[name]
    """

    @abaqus_method_doc
    def CoincidentConstraint(
        self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry
    ) -> ConstrainedSketchConstraint:
        """This method creates a coincident constraint. This constraint applies to two vertices, to a vertex and
        a ConstrainedSketchGeometry object, or to two ConstrainedSketchGeometry objects of the same type and
        constrains them to be coincident.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].CoincidentConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object or a ConstrainedSketchVertex object specifying the first object.
        entity2
            A ConstrainedSketchGeometry object or a ConstrainedSketchVertex object specifying the second object.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def ConcentricConstraint(
        self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry
    ) -> ConstrainedSketchConstraint:
        """This method creates a concentric constraint. This constraint applies to any combination of circles,
        arcs, ellipses, and points and constrains them to be concentric. A concentric constraint implies that
        the center of ConstrainedSketchGeometry objects coincide.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ConcentricConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first arc, circle, ellipse, or sketch
            vertex.
        entity2
            A ConstrainedSketchGeometry object specifying the second arc, circle, ellipse, or sketch
            vertex.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def EqualLengthConstraint(
        self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry
    ) -> ConstrainedSketchConstraint:
        """This method creates an equal length constraint. This constraint applies to lines and constrains them
        such that their lengths are equal.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].EqualLengthConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first line.
        entity2
            A ConstrainedSketchGeometry object specifying the second line.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def EqualRadiusConstraint(self, entity1: ConstrainedSketchGeometry, entity2: str) -> ConstrainedSketchConstraint:
        """This method creates an equal radius constraint. This constraint applies to circles and arcs and
        constrains them such that their radii are equal.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].EqualRadiusConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first arc or circle.
        entity2
            A ConstrainedSketchGeometry specifying the second arc or circle.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def FixedConstraint(self, entity: ConstrainedSketchGeometry) -> ConstrainedSketchConstraint:
        """This method creates a fixed constraint. This constraint applies to a ConstrainedSketchGeometry object
        or a ConstrainedSketchVertex object and constrains them to be fixed in space. Both the location and the
        shape of the sketch geometry is fixed.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].FixedConstraint

        Parameters
        ----------
        entity
            A ConstrainedSketchGeometry object or a ConstrainedSketchVertex object specifying the item to fix in
            space.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def HorizontalConstraint(self, entity: ConstrainedSketchGeometry) -> ConstrainedSketchConstraint:
        """This method creates a horizontal constraint. This constraint applies to a line and constrains it to
        be horizontal.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].HorizontalConstraint

        Parameters
        ----------
        entity
            A ConstrainedSketchGeometry object specifying the line to constrain.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def VerticalConstraint(self, entity: ConstrainedSketchGeometry) -> ConstrainedSketchConstraint:
        """This method creates a vertical constraint. This constraint applies to a line and constrains it to be
        vertical.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].VerticalConstraint

        Parameters
        ----------
        entity
            A ConstrainedSketchGeometry object specifying the line to constrain.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def ParallelConstraint(
        self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry
    ) -> ConstrainedSketchConstraint:
        """This method creates a parallel constraint. This constraint applies to lines and constrains them to be
        parallel.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].ParallelConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first line.
        entity2
            A ConstrainedSketchGeometry object specifying the second line.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def PerpendicularConstraint(
        self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry
    ) -> ConstrainedSketchConstraint:
        """This method creates a perpendicular constraint. This constraint applies to different types of
        ConstrainedSketchGeometry objects and constrains them to be perpendicular to each other.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].PerpendicularConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first object.
        entity2
            A ConstrainedSketchGeometry object specifying the second object.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def EqualDistanceConstraint(
        self, entity1: str, entity2: ConstrainedSketchGeometry, midpoint: Vertex
    ) -> ConstrainedSketchConstraint:
        """This method creates an equal distance constraint. This constraint can be applied between a midpoint
        ConstrainedSketchVertex object and any other two ConstrainedSketchVertex objects or between a midpoint
        ConstrainedSketchVertex object and two ConstrainedSketchGeometry objects that are lines. The equal
        distance constraint forces the midpoint vertex to remain at an equal distance from the two other
        vertices or lines.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].EqualDistanceConstraint

        Parameters
        ----------
        entity1
            AConstrainedSketchGeometry object specifying the first line or ConstrainedSketchVertex object.
        entity2
            A ConstrainedSketchGeometry object specifying the second line or ConstrainedSketchVertex object.
        midpoint
            A ConstrainedSketchVertex object specifying the vertex that will be positioned an equal distance from
            **entity1** and **entity2**.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()

    @abaqus_method_doc
    def TangentConstraint(
        self, entity1: ConstrainedSketchGeometry, entity2: ConstrainedSketchGeometry
    ) -> ConstrainedSketchConstraint:
        """This method creates a tangent constraint. This constraint applies to different types of
        ConstrainedSketchGeometry objects and constrains them to remain tangential.

        .. note::
            This function can be accessed by::

                mdb.models[name].sketches[name].TangentConstraint

        Parameters
        ----------
        entity1
            A ConstrainedSketchGeometry object specifying the first object.
        entity2
            A ConstrainedSketchGeometry object specifying the second object.

        Returns
        -------
        constraint: ConstrainedSketchConstraint
            A ConstrainedSketchConstraint object
        """
        return ConstrainedSketchConstraint()



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ParameterColumn:
    """The ParameterColumn object is used to define the type of parameters that will collectively build the type
    of parameter table. Once the object is created, it is noneditable.

    .. note::
        This object can be accessed by::

            mdb.models[name].tableCollections[name].parameterTables[name].columns[i]

        The corresponding analysis keywords are:

        - PARAMETER TABLE TYPE
        - PARAMETER TABLE

    .. versionadded:: 2020
        The ``ParameterColumn`` class was added.
    """

    #: A SymbolicConstant specifying the data type of the parameter. Possible values are
    #: STRING, INTEGER, and FLOAT.
    type: SymbolicConstant

    #: A String specifying the unit of the parameter.
    unit: str = ""

    #: A String specifying the description of the parameter.
    description: str = ""

    #: The default value of the first parameter. The data type of the value depends on the
    #: value of **type** argument.
    default: str = ""

    #: A Set of allowed values for the parameter.
    allowedValues: str = ""

    @abaqus_method_doc
    def __init__(
        self,
        type: Literal[C.INTEGER, C.FLOAT, C.STRING],
        unit: str = "",
        description: str = "",
        default: str = "",
        allowedValues: str = "",
    ):
        """ParameterColumn is a constructor method that creates a ParameterColumn object and stores it in the
        array data structure. It is accessible from the ParameterTable object using a column member.

        .. note::
            This function can be accessed by::

                mdb.models[name].tableCollections[name].parameterTables[name].Column

        Parameters
        ----------
        type
            A SymbolicConstant specifying the data type of the parameter. Possible values are
            STRING, INTEGER, and FLOAT.
        unit
            A String specifying the unit of the parameter.
        description
            A String specifying the description of the parameter.
        default
            The default value of the first parameter. The data type of the value depends on the
            value of **type** argument.
        allowedValues
            A Set of allowed values for the parameter.

        Returns
        -------
        ParameterTable
            A ParameterTable object.

        Raises
        ------
        Incompatible data are given
        """
        ...



from __future__ import annotations

from .AFXDataComponent import AFXDataComponent
from .FXComposite import FXComposite
from .FXIcon import FXIcon
from .FXObject import FXObject
from .FXScrollArea import FXScrollArea
from .FXTreeItem import FXTreeItem

#: Allow users to resize columns.
AFXTREETABLE_COLUMN_RESIZABLE: int = hash("AFXTREETABLE_COLUMN_RESIZABLE")

#: Disallow column selections (selecting a header/footer item in a column selects the whole column).
AFXTREETABLE_NO_COLUMN_SELECT: int = hash("AFXTREETABLE_NO_COLUMN_SELECT")

#: Selecting any item in a row selects the whole row.
AFXTREETABLE_ROW_MODE: int = hash("AFXTREETABLE_ROW_MODE")

#: Use extended selection mode that allows multiple items to be selected and allows users to drag-select a range of items.
AFXTREETABLE_EXTENDED_SELECT: int = hash("AFXTREETABLE_EXTENDED_SELECT")

#: Use single selection mode that allows up to one item to be selected.
AFXTREETABLE_SINGLE_SELECT: int = hash("AFXTREETABLE_SINGLE_SELECT")

#: Use browse selection mode that enforces one single item to be selected at all times.
AFXTREETABLE_BROWSE_SELECT: int = hash("AFXTREETABLE_BROWSE_SELECT")

#: Show item check boxes.
AFXTREETABLE_CHECK_BOXES: int = hash("AFXTREETABLE_CHECK_BOXES")

#: Propagate checked state to children and parents.
AFXTREETABLE_PROPAGATE_CHECKS: int = hash("AFXTREETABLE_PROPAGATE_CHECKS")

#: Default table options--use extended selection mode, columns are resizable, and layout fills both X and Y directions.
AFXTREETABLE_NORMAL: int = hash("AFXTREETABLE_NORMAL")

#: Do not pre-select any list item.
AFXTREETABLE_LIST_PRESELECT_NONE: int = hash("AFXTREETABLE_LIST_PRESELECT_NONE")

#: Normal list.
AFXTREETABLE_LIST_NORMAL: int = hash("AFXTREETABLE_LIST_NORMAL")


class AFXTreeTable(FXScrollArea, AFXDataComponent):
    """This class combines a tree widget with a table widget to allow associating a row of data with an item in
    a tree."""

    #: Color item has no As Is and Default in its flyout.
    COLOR_INCLUDE_COLOR_ONLY: int = hash("COLOR_INCLUDE_COLOR_ONLY")

    #: Color item has As Is in its flyout.
    COLOR_INCLUDE_AS_IS: int = hash("COLOR_INCLUDE_AS_IS")

    #: Color item has Default in its flyout.
    COLOR_INCLUDE_DEFAULT: int = hash("COLOR_INCLUDE_DEFAULT")

    #: Color item has both As Is and Default in its flyout. This is the default option.
    COLOR_INCLUDE_ALL: int = hash("COLOR_INCLUDE_ALL")

    #: Not yet implemented (Real).
    REAL: int = hash("REAL")

    #: Left justified.
    LEFT: int = hash("LEFT")

    #: Right justified.
    RIGHT: int = hash("RIGHT")

    #: Center justified (horizontal).
    CENTER: int = hash("CENTER")

    #: Top justified.
    TOP: int = hash("TOP")

    #: Bottom justified.
    BOTTOM: int = hash("BOTTOM")

    #: Middle justified (vertical).
    MIDDLE: int = hash("MIDDLE")

    #: Item accepts a text string via a text field.
    TEXT: int = hash("TEXT")

    #: Item accepts a floating-point number via a text field.
    FLOAT: int = hash("FLOAT")

    #: Item accepts an integer via a text field.
    INT: int = hash("INT")

    #: Item accepts input from a list.
    LIST: int = hash("LIST")

    #: Item is a boolean; displayed as an icon.
    BOOL: int = hash("BOOL")

    #: Item displays an icon and does not accept input.
    ICON: int = hash("ICON")

    #: Item accepts color selection via a color flyout.
    COLOR: int = hash("COLOR")

    #: General.
    GENERAL: int = hash("GENERAL")

    #: Scientific.
    SCIENTIFIC: int = hash("SCIENTIFIC")

    #: Automatic.
    AUTOMATIC: int = hash("AUTOMATIC")

    def __init__(
        self,
        p: FXComposite,
        numVisItems: int,
        numVisColumns: int,
        numColumns: int,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = AFXTREETABLE_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite
            Parent widget.
        numVisItems : int
            Number of items to display.
        numVisColumns : int
            Number of columns in the table to display.
        numColumns : int
            Number of columns in the table.
        tgt : FXObject | None
            Message target.
        sel : int
            Message ID.
        opts : int
            Options and hints.
        x : int
            X coordinate of the origin.
        y : int
            Y coordinate of the origin.
        w : int
            Width of the table widget.
        h : int
            Height of the table widget.
        """

    def addItemAfter(self, other: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Appends the new tree item after the other tree item.

        Parameters
        ----------
        other : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def addItemBefore(self, other: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Prepends the new item prior to the other tree item.

        Parameters
        ----------
        other : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def addItemFirst(self, p: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Prepends the new tree item as first child of parent.

        Parameters
        ----------
        p : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def addItemLast(self, p: FXTreeItem, item: FXTreeItem, notify: bool = False):
        """Appends the new tree item as the last child of the parent.

        Parameters
        ----------
        p : FXTreeItem

        item : FXTreeItem

        notify : bool
        """

    def addList(self, opts: int = AFXTREETABLE_LIST_NORMAL):
        """Adds a list to the table and returns the list ID. The list is used by items of type LIST.

        Parameters
        ----------
        opts : int
            List flag.
        """

    def appendListItem(self, listId: int, text: str, icon: FXIcon | None = None):
        """Appends an item to the specified table list; returns the index of the new item.

        Parameters
        ----------
        listId : int
            ID of the list to append to.
        text : str
            Item's text.
        icon : FXIcon | None
            Item's icon.
        """

    def beginEdit(self, item: FXTreeItem, column: int):
        """Sets the specified item in edit mode if the item is editable.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        """

    def cancelEdit(self):
        """Cancels the edit mode."""

    def clearContents(
        self,
        startItem: FXTreeItem,
        startColumn: int,
        endItem: FXTreeItem,
        endColumn: int,
        clearEditableOnly: bool = True,
    ):
        """Clears the text in the items in the specified range.

        Parameters
        ----------
        startItem : FXTreeItem
            Tree item in which to start clearing.
        startColumn : int
            Column in which to start clearing.
        endItem : FXTreeItem
            Tree item in which to end clearing.
        endColumn : int
            Column in which to end clearing.
        clearEditableOnly : bool
            Specify True to clear the text of editable items only.
        """

    def clearItems(self, notify: bool = False):
        """Removes all tree items and table rows.

        Parameters
        ----------
        notify : bool
        """

    def clearListItems(self, listId: int):
        """Removes all items from the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list to clear.
        """

    def closeItem(self, item: FXTreeItem, notify: bool = False):
        """Closes the specified item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def collapseTree(self, item: FXTreeItem, notify: bool = False):
        """Collapses the specified item to hide its children.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def deleteColumns(self, startColumn: int, numColumns: int = 1, notify: bool = False):
        """Deletes columns starting at the specified column.

        Parameters
        ----------
        startColumn : int
            Starting column.
        numColumns : int
            Number of columns to delete.
        notify : bool
            Specify True to notify target of the deletion.
        """

    def deselectItem(self, item: FXTreeItem, column: int, notify: bool = False):
        """Deselects the specified item.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        notify : bool
        """

    def deselectRow(self, item: FXTreeItem, notify: bool = False):
        """Deselects all items in the row.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        notify : bool
        """

    def expandTree(self, item: FXTreeItem, notify: bool = False):
        """Expands the specified item to show its children.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def getColumnWidth(self, column: int):
        """Returns the width, in pixels, of the specified column.

        Parameters
        ----------
        column : int
            Column index.
        """

    def getCurrentColumn(self):
        """Returns the column index of the current item."""

    def getCurrentItem(self):
        """Returns the current item, if any."""

    def getDefaultColumnWidth(self):
        """Returns the default column width, in pixels, of the table."""

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXScrollArea.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXScrollArea.
        """

    def getFirstItem(self):
        """Returns the first root tree item."""

    def getItemBoolValue(self, item: FXTreeItem, column: int):
        """Returns the value of a table item of type BOOL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemCheck(self, item: FXTreeItem):
        """Returns the item checked state.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemClosedIcon(self, item: FXTreeItem):
        """Returns the tree item's closed icon.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemColor(self, item: FXTreeItem, column: int):
        """Returns the color of a table item of type COLOR. The color is "As is", "Default", or a color hex
        specification in the form of "RRGGBB" (e.g., "#0A1B2C").

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemFloatValue(self, item: FXTreeItem, column: int):
        """Returns the value of a table item of type FLOAT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemFormat(self, item: FXTreeItem, column: int):
        """Returns the format of a table item of type REAL (see RealFormat).

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemIcon(self, item: FXTreeItem, column: int):
        """Returns the icon of a table item of type ICON.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemIntValue(self, item: FXTreeItem, column: int):
        """Returns the value of a table item of type INT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemListId(self, item: FXTreeItem, column: int):
        """Returns the list ID of a table item of type LIST.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemListIndex(self, item: FXTreeItem, column: int):
        """Returns the list index (selection) of a table item of type LIST.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemNumDigits(self, item: FXTreeItem, column: int):
        """Returns the number of digits to the left of the decimal point for a table item of type REAL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemOpenIcon(self, item: FXTreeItem):
        """Returns the tree item's open icon.

        Parameters
        ----------
        item : FXTreeItem
        """

    def getItemPrecision(self, item: FXTreeItem, column: int):
        """Returns the precision for a table item of type REAL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemText(self, item: FXTreeItem, column: int):
        """Returns the text of an item of type TEXT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        """

    def getItemType(self, item: FXTreeItem, column: int):
        """Returns the type of a table item.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getItemValue(self, item: FXTreeItem, column: int):
        """Returns the text-form value of a table item of any type.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def getLastItem(self):
        """Returns the last root tree item."""

    def getListItemIcon(self, listId: int, index: int):
        """Returns the icon of the item at the specified index of the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list.
        index : int
            Index into the list of the item to return.
        """

    def getListItemIndex(self, listId: int, text: str):
        """Returns the index of the item of the specified table list that has the specified text. Returns -1 if
        no such item exists.

        Parameters
        ----------
        listId : int
            ID of the list.
        text : str
            Text.
        """

    def getListItemText(self, listId: int, index: int):
        """Returns the text of the item at the specified index of the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list.
        index : int
            Index into the list of the item to return.
        """

    def getNumColumns(self):
        """Returns the number of columns."""

    def getNumItems(self):
        """Returns the number of items."""

    def getNumListItems(self, listId: int):
        """Returns the number of items in the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list.
        """

    def getTableStyle(self):
        """Returns the options related only to the table."""

    def getTreeColumn(self):
        """Returns the column index of the tree."""

    def getVisibleColumns(self):
        """Returns the number of visible columns."""

    def getVisibleItems(self):
        """Returns the number of visible items."""

    def insertColumns(self, startColumn: int, numColumns: int = 1, notify: bool = False):
        """Inserts columns at the specified location.

        Parameters
        ----------
        startColumn : int
            Starting column.
        numColumns : int
            Number of columns to insert.
        notify : bool
            Specify True to notify target of the insertion.
        """

    def isAnyItemInColumnSelected(self, column: int):
        """Returns True if any item in the column is selected.

        Parameters
        ----------
        column : int
            Column index.
        """

    def isAnyItemInRowSelected(self, item: FXTreeItem):
        """Returns True if any item in the row is selected.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        """

    def isColumnSelected(self, column: int):
        """Returns True if all items in the column are selected.

        Parameters
        ----------
        column : int
            Column index.
        """

    def isItemBool(self, item: FXTreeItem, column: int):
        """Returns True if the specified table item is of type BOOL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemColor(self, item: FXTreeItem, column: int):
        """Returns True if the specified table item is of type COLOR.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemEditable(self, item: FXTreeItem, column: int):
        """Returns True if the table item is editable.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemEmpty(self, item: FXTreeItem, column: int):
        """Returns True if the specified table item does not have a value.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemExpanded(self, item: FXTreeItem):
        """Returns True if the tree item is expanded, False otherwise.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemFloat(self, item: FXTreeItem, column: int):
        """Returns True if the specified table item is of type FLOAT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemIcon(self, item: FXTreeItem, column: int):
        """Returns True if the specified table item is of type ICON.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemInt(self, item: FXTreeItem, column: int):
        """Returns True if the specified table item is of type INT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemLeaf(self, item: FXTreeItem):
        """Returns True if the tree item is a leaf-item (has no children), False otherwise.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemList(self, item: FXTreeItem, column: int):
        """Returns True if the specified tabl eitem is of type LIST.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemOpened(self, item: FXTreeItem):
        """Returns True if the tree item is opened, False otherwise.

        Parameters
        ----------
        item : FXTreeItem
        """

    def isItemSelected(self, item: FXTreeItem, column: int):
        """Returns True if the specified item is selected.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        """

    def isItemText(self, item: FXTreeItem, column: int):
        """Returns True if the specified table item is of type TEXT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        """

    def isItemVisible(self, item: FXTreeItem, column: int):
        """Returns True if the specified item is visible.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        """

    def isRowSelected(self, item: FXTreeItem):
        """Returns True if all items in the row are selected.

        Parameters
        ----------
        item : FXTreeItem
        """

    def killSelection(self, notify: bool = False):
        """Deselects all items.

        Parameters
        ----------
        notify : bool
        """

    def makePositionVisible(self, item: FXTreeItem, column: int):
        """Scrolls to make the specified row, column fully visible.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        """

    def makeRowVisible(self, item: FXTreeItem):
        """Scrolls vertically (only) to make the specified row fully visible.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        """

    def openItem(self, item: FXTreeItem, notify: bool = False):
        """Opens the specified item.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def removeItem(self, item: FXTreeItem, notify: bool = False):
        """Removes the specified tree item and corresponding table row.

        Parameters
        ----------
        item : FXTreeItem

        notify : bool
        """

    def removeItems(self, from_: FXTreeItem, to: FXTreeItem, notify: bool = False):
        """Removes the specified tree items and their corresponding table rows, inclusively.

        Parameters
        ----------
        from_ : FXTreeItem

        to : FXTreeItem

        notify : bool
        """

    def removeListItem(self, listId: int, index: int):
        """Removes the item at the specified index from the specified table list; returns the number of items
        remaining in list.

        Parameters
        ----------
        listId : int
            ID of the list to remove from.
        index : int
            Index of the list item to remove.
        """

    def selectItem(self, item: FXTreeItem, column: int, notify: bool = False):
        """Selects the specified item.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        notify : bool
        """

    def selectRow(self, item: FXTreeItem, notify: bool = False):
        """Selects all items in the row.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        notify : bool
        """

    def setColumnBoolIcons(self, column: int, trueIcon: FXIcon | None = None, falseIcon: FXIcon | None = None):
        """Sets the True and False icons of all existing and future table items in a column of type BOOL.
        Specifying -1 for the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        trueIcon : FXIcon | None
            Icon displayed when value is True; 0 = default icon.
        falseIcon : FXIcon | None
            Icon displayed when value is False; 0 = default icon.
        """

    def setColumnBoolValue(self, column: int, value: bool):
        """Sets the value of all existing and future table items in a column of type BOOL. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        value : bool
            Specify True or False.
        """

    def setColumnColor(self, column: int, color: str):
        """Sets the color of all existing and future table items in a column of type COLOR. The color can be "As
        is", "Default", a color hex specification in the form of "RRGGBB" (e.g., "#0A1B2C"), or a pre-defined
        color name (e.g., "Red"). Specifying -1 for the column will change all columns in the table and set the
        default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        color : str
            Color.
        """

    def setColumnColorItemDefault(self, column: int, color: str):
        """Sets the color of the color item in the flyout menu for all existing and future table items that
        display "As is" or "Default" in a column of type COLOR. The color is either a color hex specification in
        the form of "RRGGBB" (e.g., "#0A1B2C") or a pre-defined color name (e.g., "Red"). Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        color : str
            Color.
        """

    def setColumnColorOptions(self, column: int, opts: int):
        """Sets the color flyout options for all existing and future table items in a column of type COLOR.
        Specifying -1 for the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        opts : int
            Options (see ColorFlyoutOptions).
        """

    def setColumnEditable(self, column: int, editable: bool):
        """Sets the editability of all existing and future table items in a column. Specifying -1 for the column
        will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        editable : bool
            Specify True for editable, False for read-only.
        """

    def setColumnFloatValue(self, column: int, value: float):
        """Sets the value of all existing and future table items in a column of type FLOAT. Specifying -1 for
        the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        value : float
            Floating-point value.
        """

    def setColumnFormat(self, column: int, format: int):
        """Sets the real format for all existing and future table items in a column of type REAL.

        Parameters
        ----------
        column : int
            Table column index.
        format : int
            Default format of REAL values in column (see RealFormat).
        """

    def setColumnIcon(self, column: int, icon: FXIcon | None = None):
        """Sets the icon of all existing and future table items in a column of type ICON. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        icon : FXIcon | None
            Icon.
        """

    def setColumnIntValue(self, column: int, value: int):
        """Sets the value of all existing and future table items in a column of type INT. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        value : int
            Integer value.
        """

    def setColumnJustify(self, column: int, justify: int):
        """Sets the justification of all existing and future table items in a column. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        justify : int
            Justification (see ItemJustify).
        """

    def setColumnListId(self, column: int, listId: int):
        """Sets the list ID of all existing and future table items in a column of type LIST. Specifying -1 for
        the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        listId : int
            List ID.
        """

    def setColumnListIndex(self, column: int, index: int):
        """Sets the list index (selection) of all existing and future table items in a column of type LIST.
        Specifying -1 for the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        index : int
            Index of item to be selected.
        """

    def setColumnNumDigits(self, column: int, numDigits: int):
        """Sets the number of digits to the left of the decimal point for all existing and future table items in
        a column of type REAL.

        Parameters
        ----------
        column : int
            Table column index.
        numDigits : int
            Default number of digits the left of the decimal point.
        """

    def setColumnPrecision(self, column: int, precision: int):
        """Sets the precision for all existing and future table items in a column of type REAL.

        Parameters
        ----------
        column : int
            Table column index.
        precision : int
            Number of digits to the right of the decimal point.
        """

    def setColumnText(self, column: int, text: str):
        """Sets the text of all existing and future table items in a column of type TEXT. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        text : str
            Text.
        """

    def setColumnType(self, column: int, type: int):
        """Sets the type of a column. Specifying -1 for the table column will change all columns in the table
        and set the default for the table.

        Parameters
        ----------
        column : int
            Table column index.
        type : int
            Type (see Flags for item types).
        """

    def setColumnWidth(self, column: int, width: int):
        """Sets the width, in pixels, of the specified column. Specifying -1 for the column will change all non-
        leading and non-trailing columns in the table and set the default for the table. Specify -1 for the
        width will resize each specified column to best fit the width of the title(s) currently shown in its
        leading and trailing items.

        Parameters
        ----------
        column : int
            Column index.
        width : int
            Width in pixels.
        """

    def setColumnWidthInChars(self, column: int, numChars: int):
        """Sets the width, in number of characters, of the specified column. Specifying -1 for the column will
        change all non-leading and non-trailing columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        numChars : int
            Width in number of characters.
        """

    def setCurrentItem(self, item: FXTreeItem, column: int, notify: bool = False):
        """Sets the current item.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        notify : bool
        """

    def setDefaultBoolIcons(self, trueIcon: FXIcon | None = None, falseIcon: FXIcon | None = None):
        """Sets the default True and False icons for the table (0 = default icon).

        Parameters
        ----------
        trueIcon : FXIcon | None
            Icon displayed when value is True; 0 = default icon.
        falseIcon : FXIcon | None
            Icon displayed when value is False; 0 = default icon.
        """

    def setDefaultBoolValue(self, value: bool):
        """Sets the default bool value for the table.

        Parameters
        ----------
        value : bool
            Specify True or False.
        """

    def setDefaultColor(self, color: str):
        """Sets the default color for all items of type COLOR in the table. The color can be "As is", "Default",
        a color hex specification in the form of "RRGGBB" (e.g., "#0A1B2C"), or a pre-defined color name (e.g.,
        "Red").

        Parameters
        ----------
        color : str
            Color.
        """

    def setDefaultColumnWidth(self, width: int):
        """Sets the default width, in pixels, for all columns.

        Parameters
        ----------
        width : int
            Width in pixels.
        """

    def setDefaultFloatValue(self, value: float):
        """Sets the default floating-point value for the table.

        Parameters
        ----------
        value : float
            Floating-point value.
        """

    def setDefaultFormat(self, format: int):
        """Sets the default real format for the table.

        Parameters
        ----------
        format : int
            Format flag.
        """

    def setDefaultIntValue(self, value: int):
        """Sets the default integer value for the table.

        Parameters
        ----------
        value : int
            Integer value.
        """

    def setDefaultJustify(self, justify: int):
        """Sets the default justification for the table.

        Parameters
        ----------
        justify : int
            Justification (see ItemJustify).
        """

    def setDefaultNumDigits(self, numDigits: int):
        """Sets the default number of digits to the left of the decimal point for the table.

        Parameters
        ----------
        numDigits : int
            Number of digits.
        """

    def setDefaultPrecision(self, precision: int):
        """Sets the precision for the table.

        Parameters
        ----------
        precision : int
            Precision.
        """

    def setDefaultText(self, text: str):
        """Sets the default text for the table.

        Parameters
        ----------
        text : str
            Text.
        """

    def setDefaultType(self, type: int):
        """Sets the default type for the table.

        Parameters
        ----------
        type : int
            Type (see Flags for item types).
        """

    def setItemBoolIcons(
        self, item: FXTreeItem, column: int, trueIcon: FXIcon | None = None, falseIcon: FXIcon | None = None
    ):
        """Sets the True and False icons of a table item of type BOOL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        trueIcon : FXIcon | None
            Icon displayed when value is True; 0 = default icon.
        falseIcon : FXIcon | None
            Icon displayed when value is False; 0 = default icon.
        """

    def setItemBoolValue(self, item: FXTreeItem, column: int, value: bool):
        """Sets the value of a table item of type BOOL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        value : bool
            Specify True or False.
        """

    def setItemCheck(self, item: FXTreeItem, check: int, notify: bool = False):
        """Sets the item checked state. Valid states are True, False and MAYBE. Returns True if the check value
        has changed, False otherwise.

        Parameters
        ----------
        item : FXTreeItem

        check : int

        notify : bool
        """

    def setItemClosedIcon(self, item: FXTreeItem, icon: FXIcon):
        """Changes the tree item's closed icon.

        Parameters
        ----------
        item : FXTreeItem

        icon : FXIcon
        """

    def setItemColor(self, item: FXTreeItem, column: int, color: str):
        """Sets the color of a table item of type COLOR. The color can be "As is", "Default", a color hex
        specification in the form of "RRGGBB" (e.g., "#0A1B2C"), or a pre-defined color name (e.g., "Red").

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        color : str
            Color.
        """

    def setItemEditable(self, item: FXTreeItem, column: int, editable: bool):
        """Sets the editability of a table item.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        editable : bool
            Specify True for editable, False for read-only.
        """

    def setItemFloatValue(self, item: FXTreeItem, column: int, value: float):
        """Sets the value of a table item of type FLOAT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        value : float
            Floating-point value.
        """

    def setItemFormat(self, item: FXTreeItem, column: int, format: int):
        """Sets the format for a table item of type REAL (see RealFormat).

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Table column index of item.
        format : int
            Format of table item (see RealFormat).
        """

    def setItemIcon(self, item: FXTreeItem, column: int, icon: FXIcon | None = None):
        """Sets the icon of a table item of type ICON.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        icon : FXIcon | None
            Icon.
        """

    def setItemIntValue(self, item: FXTreeItem, column: int, value: int):
        """Sets the value of a table item of type INT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        value : int
            Integer value.
        """

    def setItemJustify(self, item: FXTreeItem, column: int, justify: int):
        """Sets the justification of an item.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        justify : int
            Justification (see ItemJustify).
        """

    def setItemListId(self, item: FXTreeItem, column: int, listId: int):
        """Sets the list ID of a table item of type LIST.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        listId : int
            List ID.
        """

    def setItemListIndex(self, item: FXTreeItem, column: int, index: int):
        """Sets the list index (selection) of a table item of type LIST.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        index : int
            Index of item to be selected.
        """

    def setItemNumDigits(self, item: FXTreeItem, column: int, numDigits: int):
        """Sets the number of digits to the left of the decimal point for a table item of type REAL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        numDigits : int
            Number of digits.
        """

    def setItemOpenIcon(self, item: FXTreeItem, icon: FXIcon):
        """Sets the tree item's open icon.

        Parameters
        ----------
        item : FXTreeItem

        icon : FXIcon
        """

    def setItemPrecision(self, item: FXTreeItem, column: int, precision: int):
        """Sets the precision for a table item of type REAL.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        precision : int
            Number of digits to the right of the decimal point.
        """

    def setItemText(self, item: FXTreeItem, column: int, text: str):
        """Sets the text of an item of type TEXT.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of item.
        text : str
            Text.
        """

    def setItemType(self, item: FXTreeItem, column: int, type: int):
        """Sets the type of a table item.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        type : int
            Type (see Flags for item types).
        """

    def setItemValue(self, item: FXTreeItem, column: int, valueText: str):
        """Sets the value of a table item of any type that can interpret a text string for its value. Returns
        True if the value of the specified item is set successfully.

        Parameters
        ----------
        item : FXTreeItem
            Tree item.
        column : int
            Column index of table item.
        valueText : str
            Text-form value of table item.
        """

    def setLeadingRowLabels(self, str: str):
        """Set the labels of the leading row. Note: this API must be used to set the header row labels,
        otherwise labels will be overwritten by auto-numbering.

        Parameters
        ----------
        str : str
            Tab "t" delimited list, can also contain newline characters indicating that label contains multiple lines of text (e.g. "Young'snModulustPoisson'snRatio").
        """

    def setListMaxVisible(self, maxVisible: int):
        """Sets the maximum number of visible items for all table lists.

        Parameters
        ----------
        maxVisible : int
            Maximum number of visible items.
        """

    def setTableStyle(self, style: int):
        """Sets the table options.

        Parameters
        ----------
        style : int
            Style flag (see Flags for AFX table options).
        """

    def setVisibleColumns(self, visibleColumns: int):
        """Sets the number of visible columns.

        Parameters
        ----------
        visibleColumns : int
            Number of visible columns.
        """

    def setVisibleItems(self, visibleItems: int):
        """Sets the number of visible items.

        Parameters
        ----------
        visibleItems : int
            Number of visible items.
        """

    def shadeReadOnlyItems(self, shadeItems: bool):
        """Makes the table to use a different, typically shaded, background color for read-only items if True is
        passed to the method. The table would use the same regular background color for both editable and read-
        only items if False is passed to the method.

        Parameters
        ----------
        shadeItems : bool
            Specify True to use a different background color for read-only items.
        """



from __future__ import annotations

from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..BasicGeometry.VertexArray import VertexArray
from ..Datum.DatumCsys import DatumCsys
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import RIBDESIGN_REGION
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .GeometricRestriction import GeometricRestriction


@abaqus_class_doc
class TopologyRibDesign(GeometricRestriction):
    """The TopologyRibDesign object defines a topology rib design geometric restriction. The TopologyRibDesign
    object is derived from the GeometricRestriction object.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]

    .. versionadded:: 2023

        The ``TopologyRibDesign`` class was added.
    """

    #: A String specifying the geometric restriction repository key.
    name: str

    #: A VertexArray object of length 2 specifying the out-of-plane growth direction of the ribs. Instead of
    #: through a Vertex, each point can be specified through a tuple of coordinates.
    ribDirection: VertexArray

    #: A Float specifying the average thickness of the ribs.
    ribThickness: float

    #: A Float specifying the average distance between the rib centers. The distance must be larger than twice
    #: the average element edge length.
    ribDistance: float

    #: A Region object specifying the region to which the geometric restriction is applied.
    #: When used with a TopologyTask, there is no default value. When used with a ShapeTask,
    #: the default value is MODEL.
    region: Region

    #: None or a DatumCsys object specifying the position of the symmetry point defined as the
    #: origin of a local coordinate system. If **csys** = None, the global coordinate system is
    #: used. When this member is queried, it returns an Int. The default value is None.
    csys: DatumCsys | None = DatumCsys()

    #: The SymbolicConstant RIBDESIGN_REGION or a Region object specifying the overhang check region. If the value
    #: is OVERHANG_REGION, the value of region is used as both the overhang control region and the overhang check
    #: region. The default value is RIBDESIGN_REGION.
    ribDesignCheckRegion: Union[Literal[C.RIBDESIGN_REGION], Region] = RIBDESIGN_REGION

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        ribDirection: VertexArray,
        ribThickness: float,
        ribDistance: float,
        region: Region,
        csys: DatumCsys | None = None,
        ribDesignCheckRegion: Union[Literal[C.RIBDESIGN_REGION], Region] = RIBDESIGN_REGION,
    ):
        """This method creates a TopologyRibDesign object.

        .. note::
            This function can be accessed by::

                mdb.models[name].optimizationTasks[name].TopologyRibDesign

        Parameters
        ----------
        name
            A String specifying the geometric restriction repository key.
        ribDirection
            A VertexArray object of length 2 specifying the out-of-plane growth direction of the ribs. Instead of
            through a Vertex, each point can be specified through a tuple of coordinates.
        ribThickness
            A Float specifying the average thickness of the ribs.
        ribDistance
            A Float specifying the average distance between the rib centers. The distance must be larger than twice
            the average element edge length.
        region
            A Region object specifying the region to which the geometric restriction is applied.
            When used with a TopologyTask, there is no default value. When used with a ShapeTask,
            the default value is MODEL.
        csys
            None or a DatumCsys object specifying the position of the symmetry point defined as the
            origin of a local coordinate system. If **csys** = None, the global coordinate system is
            used. When this member is queried, it returns an Int. The default value is None.
        ribDesignCheckRegion
            The SymbolicConstant RIBDESIGN_REGION or a Region object specifying the overhang check region. If the value
            is OVERHANG_REGION, the value of region is used as both the overhang control region and the overhang check
            region. The default value is RIBDESIGN_REGION.

        Returns
        -------
        TopologyRibDesign
            A TopologyRibDesign object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(
        self,
        ribDirection: VertexArray,
        ribThickness: float,
        ribDistance: float,
        region: Region,
        csys: DatumCsys | None = None,
        ribDesignCheckRegion: Union[Literal[C.RIBDESIGN_REGION], Region] = RIBDESIGN_REGION,
    ):
        """This method modifies the TopologyRibDesign object.

        Parameters
        ----------
        ribDirection
            A VertexArray object of length 2 specifying the out-of-plane growth direction of the ribs. Instead of
            through a Vertex, each point can be specified through a tuple of coordinates.
        ribThickness
            A Float specifying the average thickness of the ribs.
        ribDistance
            A Float specifying the average distance between the rib centers. The distance must be larger than twice
            the average element edge length.
        region
            A Region object specifying the region to which the geometric restriction is applied.
            When used with a TopologyTask, there is no default value. When used with a ShapeTask,
            the default value is MODEL.
        csys
            None or a DatumCsys object specifying the position of the symmetry point defined as the
            origin of a local coordinate system. If **csys** = None, the global coordinate system is
            used. When this member is queried, it returns an Int. The default value is None.
        ribDesignCheckRegion
            The SymbolicConstant RIBDESIGN_REGION or a Region object specifying the overhang check region. If the value
            is OVERHANG_REGION, the value of region is used as both the overhang control region and the overhang check
            region. The default value is RIBDESIGN_REGION.
        """
        ...



from __future__ import annotations

from .AFXToolsetGui import AFXToolsetGui


class AFXProcedureToolsetGui(AFXToolsetGui):
    """This is the base class for toolset GUIs used in procedure steps (e.g. the Sketch toolset) and provides an
    interface for managing the toolset's GUI items.

    In conjuction with the AFXProcedureToolsetGuiData class, it provides a mechanism to overlay menubar,
    toolbar, and toolbox GUI items while the step executes.
    """

    #: Do not use, do not delete; for use by derived classes.
    ID_LAST: int = hash("ID_LAST")

    def __init__(self, toolsetName: str):
        """Constructor.

        Parameters
        ----------
        toolsetName : str
            Toolset name passed in from derived toolsets.
        """

    def swapInToolsetItems(self):
        """Swaps in the toolset's GUI items."""

    def swapOutToolsetItems(self):
        """Swaps out the toolset's GUI items."""
